package de.starface.examples.uci30.swing.model;

import de.starface.integration.uci.v30.client.UciProxy;
import de.starface.integration.uci.v30.client.UciProxyFactory;
import de.starface.integration.uci.v30.client.transport.UcpTransportFactory;
import de.starface.integration.uci.v30.client.transport.http.HttpUcpTransportFactory;
import de.starface.integration.uci.v30.client.transport.xmpp.XmppUcpTransportFactory;

public class UciConnectionManager implements ConnectionManager
{
	
	private UciProxy uciProxy;
	
	private Events listener;
	
	public void connect(boolean useXmpp, String xmppHost, String httpHost, String httpCallbackHost,
						String loginId, String password)
	{
		if (useXmpp)
		{
			connectUsingXmpp(xmppHost, loginId, password);
		}
		else
		{
			connectUsingHttp(httpHost, httpCallbackHost, loginId, password);
		}
	}
	
	private void connectUsingHttp(String httpHost, String httpCallbackHost, String loginId, String password)
	{
		UcpTransportFactory ucpTransportFactory = new HttpUcpTransportFactory(httpCallbackHost, 8181, httpHost,
			80, false, null, null);
		
		connectWithProxyFactory(ucpTransportFactory, loginId, password);
	}
	
	private void connectUsingXmpp(String xmppHost, String loginId, String password)
	{
		java.util.Date date = new java.util.Date();
		UcpTransportFactory ucpTransportFactory = new XmppUcpTransportFactory("UCI-3.0-Demo-Client-"
			+ date.getTime(), xmppHost, 5222, false);
		
		connectWithProxyFactory(ucpTransportFactory, loginId, password);
	}
	
	private void connectWithProxyFactory(UcpTransportFactory ucpTransportFactory, String loginId, String password)
	{
		// Close old proxy if logging in a second time
		disconnect();
		
		// Create a proxy factory
		final UciProxyFactory uciProxyFactory = UciProxyFactory.createWithTransportFactory(ucpTransportFactory);
		
		// Create a proxy with client for a STARFACE user
		uciProxy = uciProxyFactory.createUciProxy(loginId, password);
		
		// Open proxy asynchronously
		uciProxy.connectAsync();
		
		sendNewUciProxyEvents();
	}
	
	public void disconnect()
	{
		if (uciProxy != null)
		{
			uciProxy.disconnect();
		}
	}
	
	private void sendNewUciProxyEvents()
	{
		if (listener != null)
		{
			listener.newUciProxy(uciProxy);
		}
	}
	
	public void setEventListener(Events listener)
	{
		this.listener = listener;
	}
	
}
