/*
 * $Id: CallListPanel.java 980 2014-07-30 12:40:05Z dirk $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2014-07-30 14:40:05 +0200 (Mi, 30. Jul 2014) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 980 $
 */
package de.starface.examples.uci30.swing.view;

import java.awt.Dimension;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.CallListViewModel;
import de.starface.integration.uci.java.v30.values.CallListEntryDirection;
import de.starface.integration.uci.java.v30.values.CallListEntryProperties;
import de.starface.integration.uci.java.v30.values.CallListEntryResult;
import de.starface.integration.uci.java.v30.values.OrderDirection;

/**
 * This panel provides a GUI for the UCI communication call list functionality
 * 
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class CallListPanel extends JPanel
{
	
	private static final long serialVersionUID = 1L;
	
	private JTable table;
	private JScrollPane tableScrollPane;
	private JComboBox resultRestrictionCombo;
	private JComboBox orderPropertyCombo;
	private JComboBox orderDirectionCombo;
	private JFormattedTextField startAfterText;
	private JFormattedTextField countOffsetText;
	private JFormattedTextField countLimitText;
	private JLabel startBeforeLabel;
	private JLabel startAfterLabel;
	private JFormattedTextField startBeforeText;
	private JLabel directionRestrictionLabel;
	private JComboBox directionRestrictionCombo;
	private JLabel resultRestrictionLabel;
	private JLabel orderPropertyLabel;
	private JLabel orderDirectionLabel;
	private JLabel countOffsetLabel;
	private JLabel countLimitLabel;
	private JButton getCallListButton;
	private JButton deleteCallListEntryButton;
	private JButton callbackCallListEntryButton;
	private JSeparator separator;
	
	private JButton deleteCallListEntriesButton;
	
	private JLabel groupRestrictionLabel;
	
	private JComboBox groupRestrictionCombo;
	
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
	
	public CallListPanel()
	{
		initComponents();
	}
	
	private void initComponents()
	{
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		add(getOrderPropertyLabel(), new Constraints(new Leading(239, 10, 10), new Leading(12, 6, 6)));
		add(getOrderDirectionLabel(), new Constraints(new Leading(237, 30, 58), new Leading(48, 14, 6, 6)));
		add(getOrderPropertyCombo(), new Constraints(new Leading(319, 126, 30, 58), new Leading(7, 6, 6)));
		add(getOrderDirectionCombo(), new Constraints(new Leading(320, 124, 30, 58), new Leading(40, 6, 6)));
		add(getCountOffsetText(), new Constraints(new Leading(525, 40, 6, 6), new Leading(6, 6, 6)));
		add(getCountLimitText(), new Constraints(new Leading(526, 40, 10, 10), new Leading(40, 6, 6)));
		add(getCountOffsetLabel(), new Constraints(new Leading(457, 6, 6), new Leading(12, 6, 6)));
		add(getCountLimitLabel(), new Constraints(new Leading(457, 6, 6), new Leading(44, 6, 6)));
		add(getDirectionRestrictionLabel(), new Constraints(new Leading(6, 6, 6), new Leading(77, 6, 6)));
		add(getStartAfterText(), new Constraints(new Leading(72, 154, 6, 6), new Leading(6, 6, 6)));
		add(getStartBeforeText(), new Constraints(new Leading(72, 155, 6, 6), new Leading(40, 6, 6)));
		add(getStartAfterLabel(), new Constraints(new Leading(6, 6, 6), new Leading(14, 12, 6, 6)));
		add(getStartBeforeLabel(), new Constraints(new Leading(6, 6, 6), new Leading(46, 6, 6)));
		add(getJSeparator0(), new Constraints(new Leading(6, 843, 6, 6), new Leading(137, 4, 6, 6)));
		add(getDeleteCallListEntriesButton(), new Constraints(new Leading(284, 282, 6, 6), new Leading(145, 6, 6)));
		add(getDeleteCallListEntryButton(), new Constraints(new Leading(7, 271, 6, 6), new Leading(145, 6, 6)));
		add(getCallbackCallListEntryButton(), new Constraints(new Leading(572, 271, 6, 6), new Leading(145, 6, 6)));
		add(getDirectionRestrictionCombo(), new Constraints(new Leading(118, 160, 6, 6), new Leading(71, 6, 6)));
		add(getGetCallListButton(), new Constraints(new Leading(284, 282, 6, 6), new Leading(103, 6, 6)));
		add(getTableScrollPane(), new Constraints(new Bilateral(6, 10, 558), new Leading(176, 265, 10, 10)));
		add(getGroupRestrictionLabel(), new Constraints(new Leading(6, 6, 6), new Leading(108, 6, 6)));
		add(getResultRestrictionLabel(), new Constraints(new Leading(290, 6, 6), new Leading(76, 18, 6, 6)));
		add(getResultRestrictionCombo(), new Constraints(new Leading(386, 179, 6, 6), new Leading(71, 6, 6)));
		add(getGroupRestrictionCombo(), new Constraints(new Leading(118, 160, 6, 6), new Leading(103, 6, 6)));
		setSize(570, 450);
	}
	
	private JComboBox getGroupRestrictionCombo()
	{
		if (groupRestrictionCombo == null)
		{
			groupRestrictionCombo = new JComboBox();
			groupRestrictionCombo.setRenderer(new GroupSettingsListCellRenderer());
		}
		return groupRestrictionCombo;
	}
	
	private JLabel getGroupRestrictionLabel()
	{
		if (groupRestrictionLabel == null)
		{
			groupRestrictionLabel = new JLabel();
			groupRestrictionLabel.setText("groupRestriction");
		}
		return groupRestrictionLabel;
	}
	
	private JButton getDeleteCallListEntriesButton()
	{
		if (deleteCallListEntriesButton == null)
		{
			deleteCallListEntriesButton = new JButton();
			deleteCallListEntriesButton.setText("deleteCallListEntries");
		}
		return deleteCallListEntriesButton;
	}
	
	private JSeparator getJSeparator0()
	{
		if (separator == null)
		{
			separator = new JSeparator();
		}
		return separator;
	}
	
	private JButton getDeleteCallListEntryButton()
	{
		if (deleteCallListEntryButton == null)
		{
			deleteCallListEntryButton = new JButton();
			deleteCallListEntryButton.setText("deleteCallListEntry");
		}
		return deleteCallListEntryButton;
	}
	
	private JButton getCallbackCallListEntryButton()
	{
		if (callbackCallListEntryButton == null)
		{
			callbackCallListEntryButton = new JButton();
			callbackCallListEntryButton.setText("callbackCallListEntry");
		}
		return callbackCallListEntryButton;
	}
	
	private JButton getGetCallListButton()
	{
		if (getCallListButton == null)
		{
			getCallListButton = new JButton();
			getCallListButton.setText("getCallList");
		}
		return getCallListButton;
	}
	
	private JLabel getCountLimitLabel()
	{
		if (countLimitLabel == null)
		{
			countLimitLabel = new JLabel();
			countLimitLabel.setText("countLimit");
		}
		return countLimitLabel;
	}
	
	private JLabel getCountOffsetLabel()
	{
		if (countOffsetLabel == null)
		{
			countOffsetLabel = new JLabel();
			countOffsetLabel.setText("countOffset");
		}
		return countOffsetLabel;
	}
	
	private JLabel getOrderDirectionLabel()
	{
		if (orderDirectionLabel == null)
		{
			orderDirectionLabel = new JLabel();
			orderDirectionLabel.setText("orderDirection");
		}
		return orderDirectionLabel;
	}
	
	private JLabel getOrderPropertyLabel()
	{
		if (orderPropertyLabel == null)
		{
			orderPropertyLabel = new JLabel();
			orderPropertyLabel.setText("orderProperty");
		}
		return orderPropertyLabel;
	}
	
	private JLabel getResultRestrictionLabel()
	{
		if (resultRestrictionLabel == null)
		{
			resultRestrictionLabel = new JLabel();
			resultRestrictionLabel.setText("resultRestriction");
		}
		return resultRestrictionLabel;
	}
	
	private JComboBox getDirectionRestrictionCombo()
	{
		if (directionRestrictionCombo == null)
		{
			directionRestrictionCombo = new JComboBox();
		}
		return directionRestrictionCombo;
	}
	
	private JLabel getDirectionRestrictionLabel()
	{
		if (directionRestrictionLabel == null)
		{
			directionRestrictionLabel = new JLabel();
			directionRestrictionLabel.setText("directionRestriction");
		}
		return directionRestrictionLabel;
	}
	
	private JTextField getStartBeforeText()
	{
		if (startBeforeText == null)
		{
			startBeforeText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startBeforeText;
	}
	
	private JLabel getStartAfterLabel()
	{
		if (startAfterLabel == null)
		{
			startAfterLabel = new JLabel();
			startAfterLabel.setText("startAfter");
		}
		return startAfterLabel;
	}
	
	private JLabel getStartBeforeLabel()
	{
		if (startBeforeLabel == null)
		{
			startBeforeLabel = new JLabel();
			startBeforeLabel.setText("startBefore");
		}
		return startBeforeLabel;
	}
	
	private JTextField getCountLimitText()
	{
		if (countLimitText == null)
		{
			countLimitText = new JFormattedTextField();
		}
		return countLimitText;
	}
	
	private JTextField getCountOffsetText()
	{
		if (countOffsetText == null)
		{
			countOffsetText = new JFormattedTextField();
		}
		return countOffsetText;
	}
	
	private JTextField getStartAfterText()
	{
		if (startAfterText == null)
		{
			startAfterText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startAfterText;
	}
	
	private JComboBox getOrderDirectionCombo()
	{
		if (orderDirectionCombo == null)
		{
			orderDirectionCombo = new JComboBox();
		}
		return orderDirectionCombo;
	}
	
	private JComboBox getOrderPropertyCombo()
	{
		if (orderPropertyCombo == null)
		{
			orderPropertyCombo = new JComboBox();
		}
		return orderPropertyCombo;
	}
	
	private JComboBox getResultRestrictionCombo()
	{
		if (resultRestrictionCombo == null)
		{
			resultRestrictionCombo = new JComboBox();
		}
		return resultRestrictionCombo;
	}
	
	private JScrollPane getTableScrollPane()
	{
		if (tableScrollPane == null)
		{
			tableScrollPane = new JScrollPane();
			tableScrollPane.setMinimumSize(new Dimension(570, 470));
			tableScrollPane.setPreferredSize(new Dimension(570, 470));
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}
	
	private JTable getTable()
	{
		if (table == null)
		{
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][] {}, new String[] {})
			{
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[] {};
				
				public Class<?> getColumnClass(int columnIndex)
				{
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			table.setDefaultRenderer(Date.class, new DateCellRenderer());
		}
		return table;
	}
	
	public void setViewModel(CallListViewModel viewModel)
	{
		initBindings(viewModel);
	}
	
	private void initBindings(final CallListViewModel viewModel)
	{
		getCallListButton.addActionListener(viewModel.getGetCallListActionListener());
		deleteCallListEntryButton.addActionListener(viewModel.getDeleteCallListEntryActionListener());
		callbackCallListEntryButton.addActionListener(viewModel.getCallbackCallListEntryActionListener());
		
		deleteCallListEntriesButton.addActionListener(viewModel.getDeleteCallListEntriesActionListener());
		
		BeanAdapter<CallListViewModel> beanAdapter = new BeanAdapter<CallListViewModel>(viewModel, true);
		
		Bindings.bind(
			orderDirectionCombo,
			new SelectionInList<OrderDirection>(viewModel.getOrderDirectionValues(),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_ORDER_DIRECTION)));
		Bindings.bind(
			orderPropertyCombo,
			new SelectionInList<CallListEntryProperties>(viewModel.getOrderPropertyValues(),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_ORDER_PROPERTY)));
		Bindings.bind(
			directionRestrictionCombo,
			new SelectionInList<CallListEntryDirection>(viewModel.getDirectionRestrictionValues(),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_DIRECTION_RESTRICTION)));
		Bindings.bind(
			resultRestrictionCombo,
			new SelectionInList<CallListEntryResult>(viewModel.getResultRestrictionValues(),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_RESULT_RESTRICTION)));
		Bindings.bind(
			groupRestrictionCombo,
			new SelectionInList<Object>(
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_GROUP_RESTRICTION_VALUES),
				beanAdapter.getValueModel(CallListViewModel.PROPERTY_GROUP_RESTRICTION)));
		
		Bindings.bind(startAfterText, beanAdapter.getValueModel(CallListViewModel.PROPERTY_START_AFTER));
		Bindings.bind(startBeforeText, beanAdapter.getValueModel(CallListViewModel.PROPERTY_START_BEFORE));
		
		Bindings.bind(countLimitText, beanAdapter.getValueModel(CallListViewModel.PROPERTY_COUNT_LIMIT));
		Bindings.bind(countOffsetText, beanAdapter.getValueModel(CallListViewModel.PROPERTY_COUNT_OFFSET));
		
		table.setModel(viewModel.getCallListTableModel());
		table.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
		
	}
}
