/*
 * $Id: FaxPanel.java 923 2014-03-24 10:37:58Z jwiedmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2014-03-24 11:37:58 +0100 (Mo, 24. Mär 2014) $ $LastChangedBy: jwiedmann $
 * $LastChangedRevision: 923 $
 */
package de.starface.examples.uci30.swing.view;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.io.File;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.viewmodel.FaxViewModel;
import de.starface.integration.uci.java.v30.values.FaxState;

/**
 * This panel provides a GUI for the UCI communication fax functionality
 * 
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class FaxPanel extends JPanel
{
	
	private static final long serialVersionUID = 1L;
	
	private JTextField newFaxIdText;
	private JTextField faxNumberText;
	private JTextField textFileText;
	private JTextField pdfFileText;
	private JList faxStatesList;
	private JScrollPane faxStatesListScrollPane;
	private JList faxIdsList;
	private JScrollPane faxIdsListScrollPane;
	private JButton getFaxStateButton;
	private JButton openTextFileButton;
	private JButton openPdfFileButton;
	private JLabel faxIdsLabel;
	private JLabel faxStateLabel;
	private JLabel newFaxIdLabel;
	private JLabel faxNumberLabel;
	private JLabel textFileLabel;
	private JLabel pdfFileLabel;
	private JProgressBar fileProgressBar;
	private JLabel fileProgressLabel;
	private JButton sendFaxButton;
	private JButton abortFileTransferButton;
	private JLabel fileTransferStatusLabel;
	private JLabel fileTransferStatusText;
	private JButton getFaxIdsButton;
	private JLabel faxTransferStatusLabel;
	private JLabel faxTransferStatusText;
	private JLabel faxProgressLabel;
	private JProgressBar faxProgressBar;
	
	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
	
	public FaxPanel()
	{
		initComponents();
	}
	
	private void initComponents()
	{
		setLayout(new GroupLayout());
		add(getFaxIdsListScrollPane(), new Constraints(new Leading(13, 258, 10, 10), new Leading(46, 85, 10, 10)));
		add(getFaxIdsLabel(), new Constraints(new Leading(235, 6, 6), new Leading(28, 10, 10)));
		add(getGetFaxStateButton(), new Constraints(new Leading(13, 6, 6), new Leading(143, 6, 6)));
		add(getFaxStatesListScrollPane(), new Constraints(new Leading(13, 258, 6, 6), new Leading(179, 6, 6)));
		add(getNewFaxIdLabel(), new Constraints(new Leading(289, 6, 6), new Leading(52, 6, 6)));
		add(getFaxNumberLabel(), new Constraints(new Leading(289, 6, 6), new Leading(86, 6, 6)));
		add(getFaxNumberText(), new Constraints(new Leading(359, 196, 6, 6), new Leading(80, 6, 6)));
		add(getNewFaxIdText(), new Constraints(new Leading(359, 196, 6, 6), new Leading(46, 6, 6)));
		add(getFaxStateLabel(), new Constraints(new Leading(224, 6, 6), new Leading(160, 10, 10)));
		add(getOpenTextFileButton(), new Constraints(new Leading(518, 6, 6), new Leading(114, 6, 6)));
		add(getTextFileText(), new Constraints(new Leading(359, 153, 6, 6), new Leading(114, 6, 6)));
		add(getOpenPdfFileButton(), new Constraints(new Leading(518, 6, 6), new Leading(148, 6, 6)));
		add(getPdfFileLabel(), new Constraints(new Leading(289, 6, 6), new Leading(154, 6, 6)));
		add(getTextFileLabel(), new Constraints(new Leading(289, 6, 6), new Leading(120, 6, 6)));
		add(getPdfFileText(), new Constraints(new Leading(358, 154, 6, 6), new Leading(148, 6, 6)));
		add(getSendFaxButton(), new Constraints(new Leading(289, 266, 6, 6), new Leading(179, 81, 10, 10)));
		add(getAbortFileTransferButton(), new Constraints(new Leading(289, 266, 6, 6), new Leading(266, 79, 6, 6)));
		add(getGetFaxIdsButton(), new Constraints(new Leading(13, 92, 6, 6), new Leading(12, 6, 6)));
		add(getFileTransferStatusLabel(), new Constraints(new Leading(289, 6, 6), new Leading(357, 59, 59)));
		add(getFileTransferStatusText(), new Constraints(new Leading(289, 6, 6), new Leading(379, 59, 59)));
		add(getFileProgressLabel(), new Constraints(new Leading(289, 6, 6), new Leading(401, 37, 37)));
		add(getFileProgressBar(), new Constraints(new Leading(289, 266, 6, 6), new Leading(423, 6, 6)));
		add(getFaxTransferStatusLabel(), new Constraints(new Leading(19, 6, 6), new Leading(357, 6, 6)));
		add(getFaxTransferStatusText(), new Constraints(new Leading(19, 6, 6), new Leading(379, 6, 6)));
		add(getFaxProgressLabel(), new Constraints(new Leading(19, 6, 6), new Leading(401, 6, 6)));
		add(getFaxProgressBar(), new Constraints(new Leading(19, 252, 6, 6), new Leading(423, 6, 6)));
		setSize(570, 450);
	}
	
	private JProgressBar getFaxProgressBar()
	{
		if (faxProgressBar == null)
		{
			faxProgressBar = new JProgressBar();
			faxProgressBar.setStringPainted(true);
		}
		return faxProgressBar;
	}
	
	private JLabel getFaxTransferStatusLabel()
	{
		if (faxTransferStatusLabel == null)
		{
			faxTransferStatusLabel = new JLabel();
			faxTransferStatusLabel.setText("Fax pages sent / total");
		}
		return faxTransferStatusLabel;
	}
	
	private JLabel getFaxTransferStatusText()
	{
		if (faxTransferStatusText == null)
		{
			faxTransferStatusText = new JLabel();
			faxTransferStatusText.setText("0 / 0");
		}
		return faxTransferStatusText;
	}
	
	private JLabel getFaxProgressLabel()
	{
		if (faxProgressLabel == null)
		{
			faxProgressLabel = new JLabel();
			faxProgressLabel.setText("Fax sending progress");
		}
		return faxProgressLabel;
	}
	
	private JButton getGetFaxIdsButton()
	{
		if (getFaxIdsButton == null)
		{
			getFaxIdsButton = new JButton();
			getFaxIdsButton.setText("getFaxIds");
		}
		return getFaxIdsButton;
	}
	
	private JLabel getFileTransferStatusLabel()
	{
		if (fileTransferStatusLabel == null)
		{
			fileTransferStatusLabel = new JLabel();
			fileTransferStatusLabel.setText("File transfer status");
		}
		return fileTransferStatusLabel;
	}
	
	private JLabel getFileTransferStatusText()
	{
		if (fileTransferStatusText == null)
		{
			fileTransferStatusText = new JLabel();
			fileTransferStatusText.setText("No file in transfer");
		}
		return fileTransferStatusText;
	}
	
	private JButton getAbortFileTransferButton()
	{
		if (abortFileTransferButton == null)
		{
			abortFileTransferButton = new JButton();
			abortFileTransferButton.setText("abortFileTransfer");
		}
		return abortFileTransferButton;
	}
	
	private JButton getSendFaxButton()
	{
		if (sendFaxButton == null)
		{
			sendFaxButton = new JButton();
			sendFaxButton.setText("sendFaxAsync");
		}
		return sendFaxButton;
	}
	
	private JLabel getFileProgressLabel()
	{
		if (fileProgressLabel == null)
		{
			fileProgressLabel = new JLabel();
			fileProgressLabel.setText("File transfer progress");
		}
		return fileProgressLabel;
	}
	
	private JProgressBar getFileProgressBar()
	{
		if (fileProgressBar == null)
		{
			fileProgressBar = new JProgressBar();
			fileProgressBar.setStringPainted(true);
		}
		return fileProgressBar;
	}
	
	private JLabel getPdfFileLabel()
	{
		if (pdfFileLabel == null)
		{
			pdfFileLabel = new JLabel();
			pdfFileLabel.setText("pdfFile");
		}
		return pdfFileLabel;
	}
	
	private JLabel getTextFileLabel()
	{
		if (textFileLabel == null)
		{
			textFileLabel = new JLabel();
			textFileLabel.setText("textFile");
		}
		return textFileLabel;
	}
	
	private JLabel getFaxNumberLabel()
	{
		if (faxNumberLabel == null)
		{
			faxNumberLabel = new JLabel();
			faxNumberLabel.setText("faxNumber");
		}
		return faxNumberLabel;
	}
	
	private JLabel getNewFaxIdLabel()
	{
		if (newFaxIdLabel == null)
		{
			newFaxIdLabel = new JLabel();
			newFaxIdLabel.setText("faxId");
		}
		return newFaxIdLabel;
	}
	
	private JLabel getFaxStateLabel()
	{
		if (faxStateLabel == null)
		{
			faxStateLabel = new JLabel();
			faxStateLabel.setText("FaxState");
		}
		return faxStateLabel;
	}
	
	private JLabel getFaxIdsLabel()
	{
		if (faxIdsLabel == null)
		{
			faxIdsLabel = new JLabel();
			faxIdsLabel.setText("FaxIds");
		}
		return faxIdsLabel;
	}
	
	private JButton getOpenPdfFileButton()
	{
		if (openPdfFileButton == null)
		{
			openPdfFileButton = new JButton();
			openPdfFileButton.setText("...");
			openPdfFileButton.addActionListener(new ActionListener()
			{
				
				public void actionPerformed(ActionEvent event)
				{
					openPdfFileButtonActionActionPerformed(event);
				}
			});
		}
		return openPdfFileButton;
	}
	
	private JButton getOpenTextFileButton()
	{
		if (openTextFileButton == null)
		{
			openTextFileButton = new JButton();
			openTextFileButton.setText("...");
			openTextFileButton.addActionListener(new ActionListener()
			{
				
				public void actionPerformed(ActionEvent event)
				{
					openTextFileButtonActionActionPerformed(event);
				}
			});
		}
		return openTextFileButton;
	}
	
	private JButton getGetFaxStateButton()
	{
		if (getFaxStateButton == null)
		{
			getFaxStateButton = new JButton();
			getFaxStateButton.setText("getFaxState");
		}
		return getFaxStateButton;
	}
	
	private JScrollPane getFaxIdsListScrollPane()
	{
		if (faxIdsListScrollPane == null)
		{
			faxIdsListScrollPane = new JScrollPane();
			faxIdsListScrollPane.setViewportView(getFaxIdsList());
		}
		return faxIdsListScrollPane;
	}
	
	private JList getFaxIdsList()
	{
		if (faxIdsList == null)
		{
			faxIdsList = new JList();
			DefaultListModel listModel = new DefaultListModel();
			faxIdsList.setModel(listModel);
		}
		return faxIdsList;
	}
	
	private JScrollPane getFaxStatesListScrollPane()
	{
		if (faxStatesListScrollPane == null)
		{
			faxStatesListScrollPane = new JScrollPane();
			faxStatesListScrollPane.setViewportView(getFaxStatesList());
		}
		return faxStatesListScrollPane;
	}
	
	private JList getFaxStatesList()
	{
		if (faxStatesList == null)
		{
			faxStatesList = new JList();
			DefaultListModel listModel = new DefaultListModel();
			listModel.addElement("WAITING");
			listModel.addElement("PROCEEDING");
			listModel.addElement("SENDING");
			listModel.addElement("SENT");
			listModel.addElement("NOFILE");
			listModel.addElement("NOANSWER");
			listModel.addElement("BUSY");
			listModel.addElement("FAILED");
			faxStatesList.setModel(listModel);
			faxStatesList.setEnabled(false);
			faxStatesList.setFocusable(false);
		}
		return faxStatesList;
	}
	
	private JTextField getPdfFileText()
	{
		if (pdfFileText == null)
		{
			pdfFileText = new JTextField();
			pdfFileText.setText(Config.getValue("pdfFile"));
			pdfFileText.addFocusListener(new FocusAdapter()
			{
				
				public void focusGained(FocusEvent event)
				{
					pdfFileTextFocusFocusGained(event);
				}
			});
		}
		return pdfFileText;
	}
	
	private JTextField getTextFileText()
	{
		if (textFileText == null)
		{
			textFileText = new JTextField();
			textFileText.setText(Config.getValue("textFile"));
			textFileText.addFocusListener(new FocusAdapter()
			{
				
				public void focusGained(FocusEvent event)
				{
					textFileTextFocusFocusGained(event);
				}
			});
		}
		return textFileText;
	}
	
	private JTextField getFaxNumberText()
	{
		if (faxNumberText == null)
		{
			faxNumberText = new JTextField();
			faxNumberText.setText(Config.getValue("faxNumber"));
			faxNumberText.addFocusListener(new FocusAdapter()
			{
				
				public void focusGained(FocusEvent event)
				{
					faxNumberTextFocusFocusGained(event);
				}
			});
		}
		return faxNumberText;
	}
	
	private JTextField getNewFaxIdText()
	{
		if (newFaxIdText == null)
		{
			newFaxIdText = new JTextField();
			newFaxIdText.setText(Config.getValue("faxId"));
			newFaxIdText.addFocusListener(new FocusAdapter()
			{
				
				public void focusGained(FocusEvent event)
				{
					newFaxIdTextFocusFocusGained(event);
				}
			});
		}
		return newFaxIdText;
	}
	
	private void openTextFileButtonActionActionPerformed(ActionEvent event)
	{
		openTextFile();
	}
	
	private void openTextFile()
	{
		final JFileChooser fileChooser = new JFileChooser();
		FileFilter filter = new FileFilter()
		{
			@Override
			public boolean accept(File pathname)
			{
				return pathname.isDirectory() || (pathname.isFile() && pathname.getName().endsWith(".txt"));
			}
			
			@Override
			public String getDescription()
			{
				return "Text files (*.txt)";
			}
		};
		fileChooser.setFileFilter(filter);
		File file = new File(textFileText.getText());
		if (file.exists() && file.isFile())
		{
			fileChooser.setSelectedFile(file);
		}
		int returnVal = fileChooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			textFileText.setText(fileChooser.getSelectedFile().getAbsolutePath());
		}
	}
	
	private void openPdfFileButtonActionActionPerformed(ActionEvent event)
	{
		openPdfFile();
	}
	
	private void openPdfFile()
	{
		final JFileChooser fileChooser = new JFileChooser();
		FileFilter filter = new FileFilter()
		{
			@Override
			public boolean accept(File pathname)
			{
				return pathname.isDirectory() || (pathname.isFile() && pathname.getName().endsWith(".pdf"));
			}
			
			@Override
			public String getDescription()
			{
				return "PDF files (*.pdf)";
			}
		};
		fileChooser.setFileFilter(filter);
		File file = new File(pdfFileText.getText());
		if (file.exists() && file.isFile())
		{
			fileChooser.setSelectedFile(file);
		}
		int returnVal = fileChooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			pdfFileText.setText(fileChooser.getSelectedFile().getAbsolutePath());
		}
	}
	
	private void newFaxIdTextFocusFocusGained(FocusEvent event)
	{
		newFaxIdText.selectAll();
	}
	
	private void faxNumberTextFocusFocusGained(FocusEvent event)
	{
		faxNumberText.selectAll();
	}
	
	private void textFileTextFocusFocusGained(FocusEvent event)
	{
		textFileText.selectAll();
	}
	
	private void pdfFileTextFocusFocusGained(FocusEvent event)
	{
		pdfFileText.selectAll();
	}
	
	public void setViewModel(FaxViewModel viewModel)
	{
		initBindings(viewModel);
	}
	
	private void initBindings(final FaxViewModel viewModel)
	{
		getFaxIdsButton.addActionListener(viewModel.getGetFaxIdsActionListener());
		sendFaxButton.addActionListener(viewModel.getSendFaxActionListener());
		abortFileTransferButton.addActionListener(viewModel.getAbortFiletransferActionListener());
		getFaxStateButton.addActionListener(viewModel.getGetFaxStateActionListener());
		
		BeanAdapter<FaxViewModel> beanAdapter = new BeanAdapter<FaxViewModel>(viewModel, true);
		
		Bindings.bind(faxNumberText, beanAdapter.getValueModel(FaxViewModel.PROPERTY_FAX_NUMBER));
		Bindings.bind(newFaxIdText, beanAdapter.getValueModel(FaxViewModel.PROPERTY_NEW_FAX_ID));
		Bindings.bind(pdfFileText, beanAdapter.getValueModel(FaxViewModel.PROPERTY_PDF_FILE_NAME));
		Bindings.bind(textFileText, beanAdapter.getValueModel(FaxViewModel.PROPERTY_TEXT_FILE_NAME));
		
		Bindings.bind(fileTransferStatusText,
			beanAdapter.getValueModel(FaxViewModel.PROPERTY_FILE_TRANSFER_STATUS));
		Bindings.bind(faxTransferStatusText, beanAdapter.getValueModel(FaxViewModel.PROPERTY_FAX_TRANSFER_STATUS));
		
		Bindings.bind(
			faxIdsList,
			new SelectionInList<String>(beanAdapter.getValueModel(FaxViewModel.PROPERTY_FAX_IDS),
				beanAdapter.getValueModel(FaxViewModel.PROPERTY_SELECTED_FAX_ID)));
		
		Bindings.bind(
			faxStatesList,
			new SelectionInList<FaxState>(viewModel.getFaxStates(),
				beanAdapter.getValueModel(FaxViewModel.PROPERTY_FAX_STATE)));
		
		Bindings.bind(fileProgressBar, "value",
			beanAdapter.getValueModel(FaxViewModel.PROPERTY_FILE_TRANSFER_PROGRESS));
		Bindings.bind(faxProgressBar, "value",
			beanAdapter.getValueModel(FaxViewModel.PROPERTY_FAX_TRANSFER_PROGRESS));
		
	}
}
