/*
 * $Id: CallListPanel.java 499 2011-02-22 16:57:28Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2011-02-22 17:57:28 +0100 (Di, 22 Feb 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 499 $
 */
package de.starface.examples.uci30.swing.view;

import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Date;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.table.DefaultTableModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.adapter.SingleListSelectionAdapter;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.beans.PropertyConnector;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.viewmodel.FunctionKeyViewModel;

/**
 * This panel provides a GUI for the UCI communication call list functionality
 * 
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class FunctionKeyPanel extends JPanel
{
	
	private static final long serialVersionUID = 1L;
	
	private JTable table;
	private JScrollPane tableScrollPane;
	private JButton getFunctionKeysButton;
	private JButton pressFunctionKeyButton;
	
	private JButton getCallInfoForKeyButton;
	
	private JButton getContactInfoForKeyButton;
	
	private JButton getRedirectSettingsForKeyButton;
	
	private JLabel imageLabel;
	
	private JButton getImageForKeyButton;
	
	private JTable searchResultTable;
	private JScrollPane searchResultScrollPane;
	
	private JScrollPane searchIdsScrollPane;
	private JList searchIdsList;
	private JButton terminateSearchRequestButton;
	private JButton searchFunctionKeysButton;
	
	private JTextField searchIdText;
	private JTextField searchTermText;
	private JLabel searchTermLabel;
	private JLabel searchIdLabel;
	
	private JLabel keyTableLabel;
	private JLabel searchTableLabel;
	
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
	
	public FunctionKeyPanel()
	{
		initComponents();
	}
	
	private void initComponents()
	{
		setMinimumSize(new Dimension(570, 450));
		setPreferredSize(new Dimension(570, 450));
		setLayout(new GroupLayout());
		
		add(getSearchIdLabel(), new Constraints(new Leading(10, 100, 6), new Leading(10, 6, 6)));
		add(getSearchTermLabel(), new Constraints(new Leading(10, 100, 6), new Leading(40, 6, 6)));
		
		add(getSearchIdText(), new Constraints(new Leading(120, 150, 6, 6), new Leading(10, 6, 6)));
		add(getSearchTermText(), new Constraints(new Leading(120, 150, 6, 6), new Leading(40, 6, 6)));
		add(getSearchIdsScrollPane(), new Constraints(new Leading(10, 150, 6, 6), new Leading(80, 130, 6, 6)));
		
		add(getSearchFunctionKeysButton(), new Constraints(new Leading(280, 100, 6, 6), new Leading(10, 6, 6)));
		add(getTerminateSearchRequestButton(), new Constraints(new Leading(280, 100, 6, 6), new Leading(40, 6, 6)));
		
		add(getGetFunctionKeysButton(), new Constraints(new Leading(400, 100, 6, 6), new Leading(10, 6, 6)));
		add(getGetCallInfoForKeyButton(), new Constraints(new Leading(400, 100, 6, 6), new Leading(40, 6, 6)));
		
		add(getPressFunctionKeyButton(), new Constraints(new Leading(520, 100, 6, 6), new Leading(10, 6, 6)));
		add(getGetContactInfoForKeyButton(), new Constraints(new Leading(520, 100, 6, 6), new Leading(40, 6, 6)));
		
		add(getGetRedirectSettingsForKeyButton(), new Constraints(new Leading(650, 100, 6, 6), new Leading(10, 6,
			6)));
		add(getGetImageForKeyButton(), new Constraints(new Leading(650, 100, 6, 6), new Leading(40, 6, 6)));
		
		add(getImageLabel(), new Constraints(new Leading(760, 58, 186, 186), new Leading(10, 58, 6, 6)));
		
		add(getSearchTableLabel(), new Constraints(new Leading(180, 100, 6), new Leading(80, 6, 6)));
		add(getSearchResultScrollPane(), new Constraints(new Bilateral(180, 10, 500), new Leading(100, 150, 6, 6)));
		
		add(getKeyTableLabel(), new Constraints(new Leading(10, 100, 6), new Leading(240, 6, 6)));
		add(getTableScrollPane(), new Constraints(new Bilateral(10, 10, 500), new Leading(270, 150, 6, 6)));
		
		setSize(570, 450);
		
	}
	
	private JLabel getSearchTableLabel()
	{
		if (searchTableLabel == null)
		{
			searchTableLabel = new JLabel("fKey search:");
		}
		return searchTableLabel;
	}
	
	private JLabel getKeyTableLabel()
	{
		if (keyTableLabel == null)
		{
			keyTableLabel = new JLabel("function keys:");
		}
		return keyTableLabel;
	}
	
	private JLabel getSearchIdLabel()
	{
		if (searchIdLabel == null)
		{
			searchIdLabel = new JLabel("searchId");
		}
		return searchIdLabel;
	}
	
	private JLabel getSearchTermLabel()
	{
		if (searchTermLabel == null)
		{
			searchTermLabel = new JLabel("searchTerm");
		}
		return searchTermLabel;
	}
	
	private JTextField getSearchIdText()
	{
		if (searchIdText == null)
		{
			searchIdText = new JTextField("searchId");
		}
		return searchIdText;
	}
	
	private JTextField getSearchTermText()
	{
		if (searchTermText == null)
		{
			searchTermText = new JTextField("searchTerm");
		}
		return searchTermText;
	}
	
	private JList getSearchIdsList()
	{
		if (searchIdsList == null)
		{
			searchIdsList = new JList();
			DefaultListModel listModel = new DefaultListModel();
			searchIdsList.setModel(listModel);
		}
		return searchIdsList;
	}
	
	private JScrollPane getSearchIdsScrollPane()
	{
		if (searchIdsScrollPane == null)
		{
			searchIdsScrollPane = new JScrollPane();
			searchIdsScrollPane.setViewportView(getSearchIdsList());
			
		}
		return searchIdsScrollPane;
	}
	
	private JTable getSearchResultTable()
	{
		if (searchResultTable == null)
		{
			searchResultTable = new JTable();
			searchResultTable.setModel(new DefaultTableModel(new Object[][] {}, new String[] {})
			{
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[] {};
				
				public Class<?> getColumnClass(int columnIndex)
				{
					return types[columnIndex];
				}
			});
			searchResultTable.setRowSelectionAllowed(false);
			searchResultTable.setColumnSelectionAllowed(false);
			searchResultTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			searchResultTable.setDefaultRenderer(Date.class, new DateCellRenderer());
		}
		return searchResultTable;
	}
	
	private JScrollPane getSearchResultScrollPane()
	{
		if (searchResultScrollPane == null)
		{
			searchResultScrollPane = new JScrollPane();
			searchResultScrollPane.setMinimumSize(new Dimension(10, 10));
			searchResultScrollPane.setPreferredSize(new Dimension(300, 200));
			searchResultScrollPane.setViewportView(getSearchResultTable());
		}
		return searchResultScrollPane;
	}
	
	private JButton getGetImageForKeyButton()
	{
		if (getImageForKeyButton == null)
		{
			getImageForKeyButton = new JButton();
			getImageForKeyButton.setText("getImageForKey");
		}
		return getImageForKeyButton;
	}
	
	private JButton getTerminateSearchRequestButton()
	{
		if (terminateSearchRequestButton == null)
		{
			terminateSearchRequestButton = new JButton();
			terminateSearchRequestButton.setText("terminateSearchRequest");
		}
		return terminateSearchRequestButton;
	}
	
	private JButton getSearchFunctionKeysButton()
	{
		if (searchFunctionKeysButton == null)
		{
			searchFunctionKeysButton = new JButton();
			searchFunctionKeysButton.setText("searchFunctionKeys");
		}
		return searchFunctionKeysButton;
	}
	
	private JLabel getImageLabel()
	{
		if (imageLabel == null)
		{
			imageLabel = new JLabel("Img", SwingConstants.CENTER);
			imageLabel.setBorder(BorderFactory.createLineBorder(Color.black));
		}
		return imageLabel;
	}
	
	private JButton getGetRedirectSettingsForKeyButton()
	{
		if (getRedirectSettingsForKeyButton == null)
		{
			getRedirectSettingsForKeyButton = new JButton();
			getRedirectSettingsForKeyButton.setText("getRedirectSettingsForKey");
		}
		return getRedirectSettingsForKeyButton;
	}
	
	private JButton getGetContactInfoForKeyButton()
	{
		if (getContactInfoForKeyButton == null)
		{
			getContactInfoForKeyButton = new JButton();
			getContactInfoForKeyButton.setText("getContactInfoForKey");
		}
		return getContactInfoForKeyButton;
	}
	
	private JButton getGetCallInfoForKeyButton()
	{
		if (getCallInfoForKeyButton == null)
		{
			getCallInfoForKeyButton = new JButton();
			getCallInfoForKeyButton.setText("getCallInfoForKey");
		}
		return getCallInfoForKeyButton;
	}
	
	private JButton getPressFunctionKeyButton()
	{
		if (pressFunctionKeyButton == null)
		{
			pressFunctionKeyButton = new JButton();
			pressFunctionKeyButton.setText("pressFunctionKey");
		}
		return pressFunctionKeyButton;
	}
	
	private JButton getGetFunctionKeysButton()
	{
		if (getFunctionKeysButton == null)
		{
			getFunctionKeysButton = new JButton();
			getFunctionKeysButton.setText("getFunctionKeys");
		}
		return getFunctionKeysButton;
	}
	
	private JTable getTable()
	{
		if (table == null)
		{
			table = new JTable();
			table.setModel(new DefaultTableModel(new Object[][] {}, new String[] {})
			{
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[] {};
				
				public Class<?> getColumnClass(int columnIndex)
				{
					return types[columnIndex];
				}
			});
			table.setRowSelectionAllowed(true);
			table.setColumnSelectionAllowed(false);
			table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			table.setDefaultRenderer(Date.class, new DateCellRenderer());
		}
		return table;
	}
	
	private JScrollPane getTableScrollPane()
	{
		if (tableScrollPane == null)
		{
			tableScrollPane = new JScrollPane();
			tableScrollPane.setMinimumSize(new Dimension(10, 10));
			tableScrollPane.setPreferredSize(new Dimension(570, 470));
			tableScrollPane.setViewportView(getTable());
		}
		return tableScrollPane;
	}
	
	public void setViewModel(FunctionKeyViewModel viewModel)
	{
		initBindings(viewModel);
	}
	
	private void initBindings(final FunctionKeyViewModel viewModel)
	{
		PropertyChangeListener selectedRowListener = new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(FunctionKeyViewModel.PROPERTY_SELECTED_SEARCH_RESULT_ROW))
				{
					searchIdsList.setSelectedIndex(viewModel.getSelectedSearchResultRow());
				}
				
			}
		};
		
		viewModel.addPropertyChangeListener(selectedRowListener);
		
		pressFunctionKeyButton.addActionListener(viewModel.getPressFunctionKeyActionListener());
		getFunctionKeysButton.addActionListener(viewModel.getGetFunctionKeysActionListener());
		getCallInfoForKeyButton.addActionListener(viewModel.getGetCallInfoForKeysActionListener());
		getContactInfoForKeyButton.addActionListener(viewModel.getGetContactInfoForKeysActionListener());
		getRedirectSettingsForKeyButton.addActionListener(viewModel.getGetRedirectSettingsForKeysActionListener());
		getImageForKeyButton.addActionListener(viewModel.getGetImageForKeysActionListener());
		getTerminateSearchRequestButton().addActionListener(viewModel.getTerminateSearchRequestListener());
		getSearchFunctionKeysButton().addActionListener(viewModel.getSearchFunctionKeysListener());
		
		BeanAdapter<FunctionKeyViewModel> beanAdapter = new BeanAdapter<FunctionKeyViewModel>(viewModel, true);
		
		table.setModel(viewModel.getFunctionKeyTableModel());
		table.setSelectionModel(new SingleListSelectionAdapter(
			beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SELECTED_ROW)));
		table.setDefaultRenderer(String.class, new ToolTipCellRenderer(viewModel.getToolTipProvider()));
		
		searchResultTable.setModel(viewModel.getSearchResultTableModel());
		
		Bindings.bind(searchIdText, beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SEARCH_ID));
		
		Bindings.bind(searchTermText, beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SEARCH_TERM));
		
		Bindings.bind(
			searchIdsList,
			new SelectionInList<String>(
				beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SEARCH_RESULT_LIST),
				beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SELECTED_SEARCH_RESULT),
				beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_SELECTED_SEARCH_RESULT_ROW)));
		
		searchIdsList.addListSelectionListener(viewModel.getSearchResultListSelectionListener());
		
		PropertyConnector.connectAndUpdate(beanAdapter.getValueModel(FunctionKeyViewModel.PROPERTY_IMAGE),
			imageLabel, "icon");
	}
}
