/*
 * $Id: CallPanel.java 796 2013-08-09 12:23:25Z dirk $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2013-08-09 14:23:25 +0200 (Fr, 09. Aug 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 796 $
 */
package de.starface.examples.uci30.swing.view;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.SimpleDateFormat;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.DefaultTableModel;

import org.dyno.visual.swing.layouts.Bilateral;
import org.dyno.visual.swing.layouts.Constraints;
import org.dyno.visual.swing.layouts.GroupLayout;
import org.dyno.visual.swing.layouts.Leading;

import com.jgoodies.binding.adapter.Bindings;
import com.jgoodies.binding.beans.BeanAdapter;
import com.jgoodies.binding.list.SelectionInList;

import de.starface.examples.uci30.swing.view.binding.MultiListSelectionAdapter;
import de.starface.examples.uci30.swing.viewmodel.MonitorCallViewModel;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;
import de.starface.integration.uci.java.v30.values.OrderDirection;

/**
 * This panel provides a GUI for the UCI communication call functionality
 * 
 * @author tilmann
 */
// VS4E -- DO NOT REMOVE THIS LINE!
public class MonitorCallPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	private JScrollPane monitorsListScrollPane;
	private JTable monitorListEntriesTable;
	
	private JButton allMonitorsButton;
	private JButton deleteMonitorEntryButton;
	private JButton deleteMonitorEntriesButton;
	private JButton downloadMonitorEntryButton;
	private JButton createDownloadPathMonitorEntryButton;
	
	private JLabel startedAfterLabel;
	private JLabel startedBeforeLabel;
	private JLabel orderPropertyLabel;
	private JLabel orderDirectionLabel;
	private JLabel limitLabel;
	private JLabel offsetLabel;
	private JLabel monitorPathLabel;
	
	private JTextField downloadPathText;
	
	private JTextField offset;
	private JTextField limit;
	
	private String selectSingleEntry = "Select a single entry";
	private String selectMultipleEntries = "Select at least one entry";
	
	private JComboBox orderDirectionCombo;
	private JComboBox orderPropertyCombo;
	
	private JFormattedTextField startedAfterText;
	private JFormattedTextField startedBeforeText;
	
	@SuppressWarnings("unused")
	private static final String PREFERRED_LOOK_AND_FEEL = "com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel";
	
	public MonitorCallPanel()
	{
		initComponents();
	}
	
	private void initComponents()
	{
		setMinimumSize(new Dimension(850, 500));
		setPreferredSize(new Dimension(850, 500));
		setLayout(new GroupLayout());
		
		add(getMonitorsListScrollPane(), new Constraints(new Bilateral(9, 10, 500), new Leading(130, 350, 6, 6)));
		
		add(getStartedAfterLabel(), new Constraints(new Leading(9, 100, 6, 6), new Leading(6, 6, 6)));
		add(getStartedBeforeLabel(), new Constraints(new Leading(9, 100, 6, 6), new Leading(40, 6, 6)));
		add(getStartedAfterText(), new Constraints(new Leading(130, 150, 6, 6), new Leading(6, 6, 6)));
		add(getStartedBeforeText(), new Constraints(new Leading(130, 150, 6, 6), new Leading(40, 6, 6)));
		
		add(getOrderPropertyLabel(), new Constraints(new Leading(300, 100, 6), new Leading(6, 6, 6)));
		add(getOrderDirectionLabel(), new Constraints(new Leading(300, 100, 6), new Leading(40, 6, 6)));
		add(getOrderPropertyCombo(), new Constraints(new Leading(420, 120, 6, 6), new Leading(6, 6, 6)));
		add(getOrderDirectionCombo(), new Constraints(new Leading(420, 120, 6, 6), new Leading(40, 6, 6)));
		
		add(getOffsetLabel(), new Constraints(new Leading(550, 100, 6, 6), new Leading(6, 6, 6)));
		add(getLimitLabel(), new Constraints(new Leading(550, 100, 6, 6), new Leading(40, 6, 6)));
		add(getOffset(), new Constraints(new Leading(680, 40, 6, 6), new Leading(6, 6, 6)));
		add(getLimit(), new Constraints(new Leading(680, 40, 6, 6), new Leading(40, 6, 6)));
		
		add(getMonitorPathLabel(), new Constraints(new Leading(9, 100, 6, 6), new Leading(70, 6, 6)));
		add(getDownloadPathText(), new Constraints(new Leading(130, 600, 6, 6), new Leading(70, 6, 6)));
		
		add(getAllMonitorsButton(), new Constraints(new Leading(9, 150, 6, 6), new Leading(100, 6, 6)));
		add(getCreateDownloadPathMonitorEntryButton(), new Constraints(new Leading(170, 150, 6, 6), new Leading(
			100, 6, 6)));
		add(getDownloadMonitorEntryButton(), new Constraints(new Leading(330, 150, 6, 6), new Leading(100, 6, 6)));
		add(getDeleteMonitorEntryButton(), new Constraints(new Leading(490, 150, 6, 6), new Leading(100, 6, 6)));
		add(getDeleteMonitorEntriesButton(), new Constraints(new Leading(650, 150, 6, 6), new Leading(100, 6, 6)));
		
		setSize(570, 500);
	}
	
	private JLabel getMonitorPathLabel()
	{
		if (monitorPathLabel == null)
		{
			monitorPathLabel = new JLabel();
			monitorPathLabel.setText("download path");
		}
		return monitorPathLabel;
	}
	
	private JLabel getLimitLabel()
	{
		if (limitLabel == null)
		{
			limitLabel = new JLabel();
			limitLabel.setText("entries limit");
		}
		return limitLabel;
	}
	
	private JLabel getOffsetLabel()
	{
		if (offsetLabel == null)
		{
			offsetLabel = new JLabel();
			offsetLabel.setText("entries offset");
		}
		return offsetLabel;
	}
	
	private JLabel getStartedAfterLabel()
	{
		if (startedAfterLabel == null)
		{
			startedAfterLabel = new JLabel();
			startedAfterLabel.setText("start after");
		}
		return startedAfterLabel;
	}
	
	private JLabel getStartedBeforeLabel()
	{
		if (startedBeforeLabel == null)
		{
			startedBeforeLabel = new JLabel();
			startedBeforeLabel.setText("start before");
		}
		return startedBeforeLabel;
	}
	
	private JLabel getOrderPropertyLabel()
	{
		if (orderPropertyLabel == null)
		{
			orderPropertyLabel = new JLabel();
			orderPropertyLabel.setText("order by");
		}
		return orderPropertyLabel;
	}
	
	private JLabel getOrderDirectionLabel()
	{
		if (orderDirectionLabel == null)
		{
			orderDirectionLabel = new JLabel();
			orderDirectionLabel.setText("order direction");
		}
		return orderDirectionLabel;
	}
	
	private JTextField getStartedBeforeText()
	{
		if (startedBeforeText == null)
		{
			startedBeforeText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startedBeforeText;
	}
	
	private JTextField getStartedAfterText()
	{
		if (startedAfterText == null)
		{
			startedAfterText = new JFormattedTextField(new EmptyDateFormatter());
		}
		return startedAfterText;
	}
	
	private JTextField getOffset()
	{
		if (offset == null)
		{
			offset = new JTextField("0");
		}
		return offset;
	}
	
	private JTextField getLimit()
	{
		if (limit == null)
		{
			limit = new JTextField("0");
		}
		return limit;
	}
	
	private JComboBox getOrderPropertyCombo()
	{
		if (orderPropertyCombo == null)
		{
			orderPropertyCombo = new JComboBox();
		}
		return orderPropertyCombo;
	}
	
	private JComboBox getOrderDirectionCombo()
	{
		if (orderDirectionCombo == null)
		{
			orderDirectionCombo = new JComboBox();
		}
		return orderDirectionCombo;
	}
	
	private JTextField getDownloadPathText()
	{
		if (downloadPathText == null)
		{
			downloadPathText = new JTextField("");
			downloadPathText.setEditable(false);
		}
		return downloadPathText;
	}
	
	private JButton getCreateDownloadPathMonitorEntryButton()
	{
		if (createDownloadPathMonitorEntryButton == null)
		{
			createDownloadPathMonitorEntryButton = new JButton();
			createDownloadPathMonitorEntryButton.setText("Get Download Path");
			createDownloadPathMonitorEntryButton.setEnabled(false);
			createDownloadPathMonitorEntryButton.setToolTipText(selectSingleEntry);
		}
		return createDownloadPathMonitorEntryButton;
	}
	
	private JButton getDownloadMonitorEntryButton()
	{
		if (downloadMonitorEntryButton == null)
		{
			downloadMonitorEntryButton = new JButton();
			downloadMonitorEntryButton.setText("Download File");
			downloadMonitorEntryButton.setEnabled(false);
			downloadMonitorEntryButton.setToolTipText(selectSingleEntry);
		}
		return downloadMonitorEntryButton;
	}
	
	private JButton getDeleteMonitorEntryButton()
	{
		if (deleteMonitorEntryButton == null)
		{
			deleteMonitorEntryButton = new JButton();
			deleteMonitorEntryButton.setText("delete entry");
			deleteMonitorEntryButton.setEnabled(false);
			deleteMonitorEntryButton.setToolTipText(selectSingleEntry);
		}
		return deleteMonitorEntryButton;
	}
	
	private JButton getDeleteMonitorEntriesButton()
	{
		if (deleteMonitorEntriesButton == null)
		{
			deleteMonitorEntriesButton = new JButton();
			deleteMonitorEntriesButton.setText("delete entries");
			deleteMonitorEntriesButton.setEnabled(false);
			deleteMonitorEntriesButton.setToolTipText(selectMultipleEntries);
		}
		return deleteMonitorEntriesButton;
	}
	
	private JButton getAllMonitorsButton()
	{
		if (allMonitorsButton == null)
		{
			allMonitorsButton = new JButton();
			allMonitorsButton.setText("getMonitorEntries");
		}
		return allMonitorsButton;
	}
	
	private JScrollPane getMonitorsListScrollPane()
	{
		if (monitorsListScrollPane == null)
		{
			monitorsListScrollPane = new JScrollPane();
			monitorsListScrollPane.setViewportView(getMonitorListEntriesTable());
		}
		return monitorsListScrollPane;
	}
	
	private JTable getMonitorListEntriesTable()
	{
		if (monitorListEntriesTable == null)
		{
			monitorListEntriesTable = new JTable();
			monitorListEntriesTable.setModel(new DefaultTableModel(new Object[][] {}, new String[] {})
			{
				private static final long serialVersionUID = 1L;
				Class<?>[] types = new Class<?>[] {};
				
				public Class<?> getColumnClass(int columnIndex)
				{
					return types[columnIndex];
				}
			});
			monitorListEntriesTable.setRowSelectionAllowed(true);
			monitorListEntriesTable.setColumnSelectionAllowed(false);
		}
		return monitorListEntriesTable;
	}
	
	private PropertyChangeListener enableGetDownloadPathButton = new PropertyChangeListener()
	{
		public void propertyChange(PropertyChangeEvent e)
		{
			if (e.getPropertyName() == MonitorCallViewModel.GET_DOWNLOAD_PATH_IS_ALLOWED)
			{
				boolean enableButton = (Boolean) e.getNewValue();
				getCreateDownloadPathMonitorEntryButton().setEnabled(enableButton);
				getCreateDownloadPathMonitorEntryButton().setToolTipText(enableButton ? null : selectSingleEntry);
			}
		}
	};
	
	private PropertyChangeListener enableDownloadFileButton = new PropertyChangeListener()
	{
		public void propertyChange(PropertyChangeEvent e)
		{
			if (e.getPropertyName() == MonitorCallViewModel.DOWNLOAD_ENTRY_IS_ALLOWED)
			{
				boolean enableButton = (Boolean) e.getNewValue();
				getDownloadMonitorEntryButton().setEnabled(enableButton);
				getDownloadMonitorEntryButton().setToolTipText(enableButton ? null : selectSingleEntry);
			}
		}
	};
	
	private PropertyChangeListener enableDeleteEntryButton = new PropertyChangeListener()
	{
		public void propertyChange(PropertyChangeEvent e)
		{
			if (e.getPropertyName() == MonitorCallViewModel.DELETE_ENTRY_IS_ALLOWED)
			{
				boolean enableButton = (Boolean) e.getNewValue();
				getDeleteMonitorEntryButton().setEnabled(enableButton);
				getDeleteMonitorEntryButton().setToolTipText(enableButton ? null : selectSingleEntry);
			}
		}
	};
	
	private PropertyChangeListener enableDeleteEntriesButton = new PropertyChangeListener()
	{
		public void propertyChange(PropertyChangeEvent e)
		{
			if (e.getPropertyName() == MonitorCallViewModel.DELETE_ENTRIES_IS_ALLOWED)
			{
				boolean enableButton = (Boolean) e.getNewValue();
				getDeleteMonitorEntriesButton().setEnabled(enableButton);
				getDeleteMonitorEntriesButton().setToolTipText(enableButton ? null : selectMultipleEntries);
			}
		}
	};
	
	public void setViewModel(MonitorCallViewModel viewModel)
	{
		initBindings(viewModel);
	}
	
	private void initBindings(final MonitorCallViewModel viewModel)
	{
		getAllMonitorsButton().addActionListener(viewModel.getAllMonitorsActionListener());
		getDeleteMonitorEntryButton().addActionListener(viewModel.getDeleteMonitorEntryActionListener());
		getDeleteMonitorEntriesButton().addActionListener(viewModel.getDeleteMonitorEntriesActionListener());
		getDownloadMonitorEntryButton().addActionListener(viewModel.getDownloadMonitorEntryActionListener());
		getCreateDownloadPathMonitorEntryButton().addActionListener(
			viewModel.getGetDownloadFileUrlMonitorEntryActionListener());
		
		BeanAdapter<MonitorCallViewModel> beanAdapter = new BeanAdapter<MonitorCallViewModel>(viewModel, true);
		
		viewModel.addPropertyChangeListener(enableDeleteEntriesButton);
		viewModel.addPropertyChangeListener(enableDeleteEntryButton);
		viewModel.addPropertyChangeListener(enableDownloadFileButton);
		viewModel.addPropertyChangeListener(enableGetDownloadPathButton);
		
		Bindings.bind(downloadPathText, beanAdapter.getValueModel(MonitorCallViewModel.DOWNLOAD_PATH_MONITOR));
		Bindings.bind(limit, beanAdapter.getValueModel(MonitorCallViewModel.LIMIT_MONITOR));
		Bindings.bind(offset, beanAdapter.getValueModel(MonitorCallViewModel.OFFSET_MONITOR));
		Bindings.bind(startedAfterText, beanAdapter.getValueModel(MonitorCallViewModel.STARTED_AFTER_MONITOR));
		Bindings.bind(startedBeforeText, beanAdapter.getValueModel(MonitorCallViewModel.STARTED_BEFORE_MONITOR));
		Bindings.bind(
			orderDirectionCombo,
			new SelectionInList<OrderDirection>(viewModel.getOrderDirectionMonitorValues(),
				beanAdapter.getValueModel(MonitorCallViewModel.PROPERTY_ORDER_DIRECTION_MONITOR)));
		
		Bindings.bind(orderPropertyCombo,
			new SelectionInList<MonitorListEntryProperties>(viewModel.getMonitorListEntryPropertiesValues(),
				beanAdapter.getValueModel(MonitorCallViewModel.MONITOR_LIST_ENTRY_PROPERTIES)));
		
		class myDateCellRenderer extends DefaultTableCellRenderer
		{
			public myDateCellRenderer()
			{
				super();
			}
			
			@Override
			public void setValue(Object value)
			{
				SimpleDateFormat sdf = new SimpleDateFormat("H:m MMM d, yyyy");
				
				setText((value == null) ? "" : sdf.format(value));
			}
		}
		
		monitorListEntriesTable.setModel(viewModel.getMonitorListTableModel());
		monitorListEntriesTable.setSelectionModel(new MultiListSelectionAdapter(viewModel.getSelectedRows()));
		monitorListEntriesTable.getSelectionModel().addListSelectionListener(viewModel.getSelectedRowsListener());
		monitorListEntriesTable.removeColumn(monitorListEntriesTable.getColumn(MonitorListEntryProperties.file.toString()));
		monitorListEntriesTable.getColumn(MonitorListEntryProperties.monitortimestart.toString()).setCellRenderer(
			new myDateCellRenderer());
		monitorListEntriesTable.getColumn(MonitorListEntryProperties.monitortimeend.toString()).setCellRenderer(
			new myDateCellRenderer());
		
	}
}
