/*
 * $Id: RedirectSettingsDestinationCellRenderer.java 851 2013-11-20 11:23:28Z dirk $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2013-11-20 12:23:28 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 851 $
 */
package de.starface.examples.uci30.swing.view;

import java.awt.Component;
import java.util.List;

import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;

import de.starface.examples.uci30.swing.viewmodel.RedirectSettingsTableModel;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.values.RedirectSettingDestinationType;

/**
 * A table cell renderer for the destination property of redirect settings
 * 
 * @author tilmann
 */
public class RedirectSettingsDestinationCellRenderer extends DefaultTableCellRenderer
{
	
	private static final long serialVersionUID = 1L;
	
	public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected,
												   boolean hasFocus, int row, int column)
	{
		final RedirectSetting redirectSetting = ((RedirectSettingsTableModel) table.getModel()).getRedirectSetting(row);
		final RedirectSettingDestinationType destinationType = redirectSetting.getDestinationType();
		
		if (destinationType == RedirectSettingDestinationType.VOICEMAIL)
		{
			final List<Mailbox> mailboxes = redirectSetting.getMailboxes();
			value = computeSelectedItemName((String) value, mailboxes);
		}
		return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	}
	
	static Object computeSelectedItemName(String value, List<Mailbox> mailboxes)
	{
		if (value == null) return null;
		for (Mailbox mailbox : mailboxes)
		{
			if (value.equals(mailbox.getId()))
			{
				return mailbox.getName();
			}
		}
		return null;
	}
}
