package de.starface.examples.uci30.swing.view.binding;

import java.util.Collections;
import java.util.List;

import javax.swing.DefaultListSelectionModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class MultiListSelectionAdapter implements ListSelectionModel
{
	private DefaultListSelectionModel defaultSelectionModel = new DefaultListSelectionModel();
	
	private DefaultSelectionModelListener defaultSelectionModelListener = new DefaultSelectionModelListener();
	
	private List<Integer> selection;
	
	public MultiListSelectionAdapter(List<Integer> selection)
	{
		this.selection = selection;
		defaultSelectionModel.setSelectionMode(DefaultListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		defaultSelectionModel.addListSelectionListener(defaultSelectionModelListener);
	}
	
	private class DefaultSelectionModelListener implements ListSelectionListener
	{
		@Override
		public void valueChanged(ListSelectionEvent e)
		{
			defaultSelectionModel.removeListSelectionListener(defaultSelectionModelListener);
			ListSelectionModel source = (ListSelectionModel) e.getSource();
			for (int index = e.getFirstIndex(); index <= e.getLastIndex(); index++)
			{
				Integer indexInt = (Integer) index;
				if (selection.contains(indexInt) && !source.isSelectedIndex(index))
				{
					selection.remove(indexInt);
				}
				else if (!selection.contains(indexInt) && source.isSelectedIndex(index))
				{
					selection.add(indexInt);
				}
			}
			Collections.sort(selection);
			defaultSelectionModel.addListSelectionListener(defaultSelectionModelListener);
		}
	}
	
	@Override
	public void setSelectionInterval(int index0, int index1)
	{
		defaultSelectionModel.setSelectionInterval(index0, index1);
	}
	
	@Override
	public void addSelectionInterval(int index0, int index1)
	{
		defaultSelectionModel.addSelectionInterval(index0, index1);
	}
	
	@Override
	public void removeSelectionInterval(int index0, int index1)
	{
		defaultSelectionModel.removeSelectionInterval(index0, index1);
	}
	
	@Override
	public int getMinSelectionIndex()
	{
		return defaultSelectionModel.getMinSelectionIndex();
	}
	
	@Override
	public int getMaxSelectionIndex()
	{
		return defaultSelectionModel.getMaxSelectionIndex();
	}
	
	@Override
	public boolean isSelectedIndex(int index)
	{
		return defaultSelectionModel.isSelectedIndex(index);
	}
	
	@Override
	public int getAnchorSelectionIndex()
	{
		return defaultSelectionModel.getAnchorSelectionIndex();
	}
	
	@Override
	public void setAnchorSelectionIndex(int index)
	{
		defaultSelectionModel.setAnchorSelectionIndex(index);
	}
	
	@Override
	public int getLeadSelectionIndex()
	{
		return defaultSelectionModel.getLeadSelectionIndex();
	}
	
	@Override
	public void setLeadSelectionIndex(int index)
	{
		defaultSelectionModel.setLeadSelectionIndex(index);
	}
	
	@Override
	public void clearSelection()
	{
		defaultSelectionModel.clearSelection();
	}
	
	@Override
	public boolean isSelectionEmpty()
	{
		return defaultSelectionModel.isSelectionEmpty();
	}
	
	@Override
	public void insertIndexInterval(int index, int length, boolean before)
	{
		defaultSelectionModel.insertIndexInterval(index, length, before);
	}
	
	@Override
	public void removeIndexInterval(int index0, int index1)
	{
		defaultSelectionModel.removeIndexInterval(index0, index1);
	}
	
	@Override
	public void setValueIsAdjusting(boolean valueIsAdjusting)
	{
		defaultSelectionModel.setValueIsAdjusting(valueIsAdjusting);
	}
	
	@Override
	public boolean getValueIsAdjusting()
	{
		return defaultSelectionModel.getValueIsAdjusting();
	}
	
	@Override
	public void setSelectionMode(int selectionMode)
	{
		defaultSelectionModel.setSelectionMode(selectionMode);
	}
	
	@Override
	public int getSelectionMode()
	{
		return defaultSelectionModel.getSelectionMode();
	}
	
	@Override
	public void addListSelectionListener(ListSelectionListener x)
	{
		defaultSelectionModel.addListSelectionListener(x);
	}
	
	@Override
	public void removeListSelectionListener(ListSelectionListener x)
	{
		defaultSelectionModel.removeListSelectionListener(x);
	}
	
}
