package de.starface.examples.uci30.swing.viewmodel;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.starface.integration.uci.java.v30.types.Call;
import de.starface.integration.uci.java.v30.values.CallProperties;

public class CallDetailsTableModel implements TableModel
{
	
	private static final String[] columnOrder = new String[2];
	
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();
	
	private static final CallProperties[] callKeys = new CallProperties[CallProperties.values().length];
	private Call call;
	
	static
	{
		columnOrder[0] = "keys";
		columnOrder[1] = "values";
		
		callKeys[0] = CallProperties.id;
		callKeys[1] = CallProperties.state;
		callKeys[2] = CallProperties.timestamp;
		callKeys[3] = CallProperties.calledNumber;
		callKeys[4] = CallProperties.calledName;
		callKeys[5] = CallProperties.callerNumber;
		callKeys[6] = CallProperties.callerName;
		callKeys[7] = CallProperties.groupId;
		callKeys[8] = CallProperties.peerNames;
		callKeys[9] = CallProperties.channelNames;
		callKeys[10] = CallProperties.sipCallIds;
		callKeys[11] = CallProperties.caller;
		callKeys[12] = CallProperties.connectedTimestamp;
		callKeys[13] = CallProperties.recorder;
		callKeys[14] = CallProperties.recorded;
		callKeys[15] = CallProperties.recordOwner;
		callKeys[16] = CallProperties.avatarHash;
		callKeys[17] = CallProperties.doorlineCamUrl;
		callKeys[18] = CallProperties.doorlineDtmfCode;
		callKeys[19] = CallProperties.doorlineImageProviderId;
		callKeys[20] = CallProperties.forwardType;
		callKeys[21] = CallProperties.forwarderCallerIdName;
		callKeys[22] = CallProperties.forwarderCallerIdNumber;
		callKeys[23] = CallProperties.conferenceRoomId;
		callKeys[24] = CallProperties.conferenceAdmin;
		callKeys[25] = CallProperties.referenceOfConsultation;
		callKeys[26] = CallProperties.duration;
		callKeys[27] = CallProperties.consultationCall;
		callKeys[28] = CallProperties.jabberId;
	}
	
	public CallDetailsTableModel()
	{
		call = null;
	}
	
	@Override
	public int getRowCount()
	{
		return callKeys.length;
	}
	
	@Override
	public int getColumnCount()
	{
		return columnOrder.length;
	}
	
	@Override
	public String getColumnName(int columnIndex)
	{
		return columnOrder[columnIndex];
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex)
	{
		return Object.class;
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	
	@Override
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			case 0:
				return callKeys[rowIndex].toString();
			case 1:
			default:
				if (call == null)
				{
					return "";
				}
				else
				{
					return Utils.getProperty(callKeys[rowIndex], call);
				}
		}
	}
	
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		// not required
	}
	
	@Override
	public void addTableModelListener(TableModelListener listener)
	{
		listeners.add(listener);
	}
	
	@Override
	public void removeTableModelListener(TableModelListener listener)
	{
		listeners.remove(listener);
	}
	
	private void fireTableChanged(TableModelEvent event)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(event);
			}
		}
	}
	
	public void updateCallDetails(Call call)
	{
		this.call = call;
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void clearCallDetails()
	{
		this.call = null;
		fireTableChanged(new TableModelEvent(this));
	}
}
