package de.starface.examples.uci30.swing.viewmodel;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.starface.integration.uci.java.v30.types.Call;
import de.starface.integration.uci.java.v30.values.CallProperties;

public class CallTableModel implements TableModel
{
	
	private static final CallProperties[] columnOrder = new CallProperties[CallProperties.values().length];
	
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();
	
	private ArrayList<Call> calls;
	
	static
	{
		columnOrder[0] = CallProperties.id;
		columnOrder[1] = CallProperties.state;
		columnOrder[2] = CallProperties.timestamp;
		columnOrder[3] = CallProperties.calledNumber;
		columnOrder[4] = CallProperties.calledName;
		columnOrder[5] = CallProperties.callerNumber;
		columnOrder[6] = CallProperties.callerName;
		columnOrder[7] = CallProperties.groupId;
		columnOrder[8] = CallProperties.peerNames;
		columnOrder[9] = CallProperties.channelNames;
		columnOrder[10] = CallProperties.sipCallIds;
		columnOrder[11] = CallProperties.caller;
		columnOrder[12] = CallProperties.connectedTimestamp;
		columnOrder[13] = CallProperties.recorder;
		columnOrder[14] = CallProperties.recorded;
		columnOrder[15] = CallProperties.recordOwner;
		columnOrder[16] = CallProperties.avatarHash;
		columnOrder[17] = CallProperties.doorlineCamUrl;
		columnOrder[18] = CallProperties.doorlineDtmfCode;
		columnOrder[19] = CallProperties.doorlineImageProviderId;
		columnOrder[20] = CallProperties.forwardType;
		columnOrder[21] = CallProperties.forwarderCallerIdName;
		columnOrder[22] = CallProperties.forwarderCallerIdNumber;
		columnOrder[23] = CallProperties.conferenceRoomId;
		columnOrder[24] = CallProperties.conferenceAdmin;
		columnOrder[25] = CallProperties.referenceOfConsultation;
		columnOrder[26] = CallProperties.duration;
		columnOrder[27] = CallProperties.consultationCall;
		columnOrder[28] = CallProperties.jabberId;
	}
	
	public CallTableModel()
	{
		calls = new ArrayList<Call>();
	}
	
	@Override
	public int getRowCount()
	{
		return calls.size();
	}
	
	@Override
	public int getColumnCount()
	{
		return columnOrder.length;
	}
	
	@Override
	public String getColumnName(int columnIndex)
	{
		return columnOrder[columnIndex].name();
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex)
	{
		return columnOrder[columnIndex].getType();
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		return false;
	}
	
	@Override
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		if (rowIndex > calls.size() - 1)
		{
			return null;
		}
		return Utils.getProperty(columnOrder[columnIndex], calls.get(rowIndex));
		
	}
	
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		// TODO Auto-generated method stub
		// not required
	}
	
	@Override
	public void addTableModelListener(TableModelListener listener)
	{
		listeners.add(listener);
	}
	
	@Override
	public void removeTableModelListener(TableModelListener listener)
	{
		listeners.remove(listener);
	}
	
	private void fireTableChanged(TableModelEvent event)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(event);
			}
		}
	}
	
	public void updateCall(Call call)
	{
		String id = call.getId();
		int rowIndex = 0;
		for (ListIterator<Call> listIterator = calls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex())
		{
			if (listIterator.next().getId().equals(id))
			{
				listIterator.set(call);
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.UPDATE));
				break;
			}
		}
		if (rowIndex == calls.size())
		{
			calls.add(call);
			fireTableChanged(new TableModelEvent(this, calls.size(), calls.size(), TableModelEvent.ALL_COLUMNS,
				TableModelEvent.INSERT));
		}
	}
	
	public void removeCall(String callId)
	{
		int rowIndex = 0;
		for (ListIterator<Call> listIterator = calls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex())
		{
			if (listIterator.next().getId().equals(callId))
			{
				listIterator.remove();
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.DELETE));
			}
		}
	}
	
	public Call getCall(String callId)
	{
		for (Call call : calls)
		{
			if (call.getId().equals(callId))
			{
				return call;
			}
		}
		return null;
	}
	
	public void changeCallId(String oldCallId, String newCallId)
	{
		int rowIndex = 0;
		for (ListIterator<Call> listIterator = calls.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex())
		{
			Call call = listIterator.next();
			if (call.getId().equals(oldCallId))
			{
				call.setId(newCallId);
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.DELETE));
			}
		}
	}
	
	public void removeAllCalls()
	{
		calls.clear();
		fireTableChanged(new TableModelEvent(this));
	}
	
	public Call getCall(int rowIndex)
	{
		return calls.get(rowIndex);
	}
}
