package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciCcbsCallEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciCcbsCallRequests;
import de.starface.integration.uci.java.v30.types.CcbsCall;
import de.starface.integration.uci.java.v30.values.CcbsAccountState;
import de.starface.integration.uci.java.v30.values.CcbsCallState;
import de.starface.integration.uci.v30.client.UciServiceRequests;

public class CcbsCallViewModel implements UciCcbsCallEvents
{
	
	private UciCcbsCallRequests callRequests;
	
	public CcbsCallViewModel()
	{
	}
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.callRequests = uciServiceRequests.getRequests(UciCcbsCallRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciCcbsCallEvents.class);
	}
	
	@Override
	@RpcAsynchronous
	public void receiveCcbsAccountState(CcbsAccountState ccbsAccountState)
	{
		setCcbsStatus(ccbsAccountState.toString());
	}
	
	@Override
	@RpcAsynchronous
	public void receiveCcbsCallState(CcbsCall ccbsCall)
	{
		if (ccbsCall != null)
		{
			if (ccbsCall.getCcbsCallState() == CcbsCallState.INACTIVE)
			{
				removeCcbsCall(ccbsCall.getId());
			}
			else
			{
				updateCcbsCall(ccbsCall);
			}
		}
	}
	
	private void removeCcbsCall(String callId)
	{
		if (selectedCall != null && selectedCall.getId().equals(callId))
		{
			setSelectedCall(null);
		}
		ccbsCallTableModel.removeCcbsCall(callId);
	}
	
	private void updateCcbsCall(CcbsCall call)
	{
		if (selectedCall != null && selectedCall.getId().equals(call.getId()))
		{
			setSelectedCall(call);
		}
		ccbsCallTableModel.updateCcbsCall(call);
	}
	
	private void doGetAllCcbsCalls()
	{
		try
		{
			// Retrieve all current CcbsCalls
			List<CcbsCall> ccbsCalls = callRequests.getAllCcbsCalls();
			
			for (CcbsCall call : ccbsCalls)
			{
				if (call != null)
				{
					if (call.getCcbsCallState() == CcbsCallState.INACTIVE)
					{
						removeCcbsCall(call.getId());
					}
					else
					{
						updateCcbsCall(call);
					}
				}
			}
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	public void doGetCcbsAccountState()
	{
		try
		{
			// Get CcbsAccountState
			setCcbsStatus(callRequests.getCcbsAccountState().toString());
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	public void doRemoveAvailableCcbs()
	{
		if (selectedCall != null)
		{
			try
			{
				// Ccbs start or stop
				callRequests.removeAvailableCcbs(selectedCall.getId());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doCcbs()
	{
		
		if (selectedCall != null)
		{
			try
			{
				// Ccbs start or stop
				callRequests.ccbs(selectedCall.getId());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	private CcbsCallTableModel ccbsCallTableModel = new CcbsCallTableModel();
	
	private List<Integer> selectedRows = new ArrayList<Integer>();
	
	private CcbsCall selectedCall = null;
	
	private String ccbsStatus = "";
	
	private boolean ccbsIsAllowed = false;
	
	public TableModel getCcbsCallTableModel()
	{
		return ccbsCallTableModel;
	}
	
	public List<Integer> getSelectedRows()
	{
		return selectedRows;
	}
	
	public static final String PROPERTY_CCBS_STATUS = "ccbsStatus";
	
	public String getCcbsStatus()
	{
		return ccbsStatus;
	}
	
	public void setCcbsStatus(String ccbsStatus)
	{
		String old = this.ccbsStatus;
		this.ccbsStatus = ccbsStatus;
		changeSupport.firePropertyChange(PROPERTY_CCBS_STATUS, old, ccbsStatus);
	}
	
	public static final String PROPERTY_GET_CCBS_IS_ALLOWED = "ccbsIsAllowed";
	
	public boolean getCcbsIsAllowed()
	{
		return ccbsIsAllowed;
	}
	
	public void setCcbsIsAllowed(boolean ccbsIsAllowed)
	{
		boolean old = this.ccbsIsAllowed;
		this.ccbsIsAllowed = ccbsIsAllowed;
		changeSupport.firePropertyChange(PROPERTY_GET_CCBS_IS_ALLOWED, old, ccbsIsAllowed);
		
	}
	
	public CcbsCall getSelectedCall()
	{
		return selectedCall;
	}
	
	public void setSelectedCall(CcbsCall selectedCall)
	{
		this.selectedCall = selectedCall;
	}
	
	public ListSelectionListener getSelectedCallsListener()
	{
		return selectedCallsListSelectionListener;
	}
	
	private final ListSelectionListener selectedCallsListSelectionListener = new ListSelectionListener()
	{
		public void valueChanged(ListSelectionEvent e)
		{
			if (selectedRows.size() == 1)
			{
				setSelectedCall(ccbsCallTableModel.getCcbsCall(selectedRows.get(0)));
				setCcbsIsAllowed(true);
			}
			else
			{
				setSelectedCall(null);
				setCcbsIsAllowed(false);
			}
		}
	};
	
	public ActionListener getCcbsCallActionListener()
	{
		return ccbsCallActionListener;
	}
	
	public ActionListener getRemoveAvailableCcbsActionListener()
	{
		return removeAvailableCcbsActionListener;
	}
	
	private ActionListener ccbsCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCcbs();
		}
	};
	
	private ActionListener removeAvailableCcbsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doRemoveAvailableCcbs();
		}
	};
	
	public ActionListener getAccountStateActionListener()
	{
		return accountStateActionListener;
	}
	
	private ActionListener accountStateActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetCcbsAccountState();
		}
	};
	
	public ActionListener getAllCcbsCallsActionListener()
	{
		return allCcbsCallsActionListener;
	}
	
	private ActionListener allCcbsCallsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetAllCcbsCalls();
		}
	};
	
}
