package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.List;

import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import org.apache.commons.lang.StringUtils;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciConferenceCallEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciConferenceCallRequests;
import de.starface.integration.uci.java.v30.types.Call;
import de.starface.integration.uci.java.v30.types.ConferenceCall;
import de.starface.integration.uci.java.v30.values.CallState;
import de.starface.integration.uci.v30.client.UciServiceRequests;

public class ConferenceCallViewModel implements UciConferenceCallEvents
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private UciConferenceCallRequests callRequests;
	
	private PhoneToUseProvider phoneToUseProvider;
	
	public ConferenceCallViewModel(PhoneToUseProvider phoneToUseProvider)
	{
		this.phoneToUseProvider = phoneToUseProvider;
	}
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.callRequests = uciServiceRequests.getRequests(UciConferenceCallRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciConferenceCallEvents.class);
	}
	
	@Override
	public void receiveConferenceCallState(ConferenceCall call)
	{
		if (call != null)
		{
			if (call.getState() == CallState.HANGUP)
			{
				removeConferenceCall(call);
			}
			else
			{
				updateConferenceCall(call);
			}
		}
		
	}
	
	public void receiveCallState(Call call)
	{
		if (call != null)
		{
			if (call.getState() == CallState.HANGUP)
			{
				removeCall(call.getId());
			}
			else
			{
				updateCall(call);
			}
		}
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void doGetAllConferenceIds()
	{
		try
		{
			// Gets all current conference Ids
			List<String> conferences = callRequests.getAllConferences();
			setConferenceIds(conferences);
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doGetAllConferenceCalls()
	{
		try
		{
			if (!StringUtils.isEmpty(selectedConferenceId))
			{
				// Gets all calls in the selected conference
				List<ConferenceCall> conferenceCalls = callRequests.getAllConferenceCallsInRoom(selectedConferenceId);
				conferenceCallTableModel.setConferenceCalls(conferenceCalls);
			}
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	private void doHangupCall()
	{
		if (!StringUtils.isEmpty(selectedConferenceCallId))
		{
			try
			{
				// Hangup call sending hangupCall request
				callRequests.hangupCall(selectedConferenceCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	//	private void doMediateCallForConference()
	//	{
	//		
	//		if (!StringUtils.isEmpty(selectedConferenceId) && !StringUtils.isEmpty(destinationNumber))
	//		{
	//			try
	//			{
	//				// start new call for mediation into the selected conference
	//				callRequests.mediateNumberForConference(selectedConferenceId, destinationNumber);
	//			}
	//			catch (UciException ex)
	//			{
	//				// Too simple error management. MUST be improved in real client!
	//				ex.printStackTrace();
	//			}
	//		}
	//	}
	
	private void doTransferCallForConference()
	{
		
		if (!StringUtils.isEmpty(selectedConferenceId) && !StringUtils.isEmpty(selectedCallId))
		{
			try
			{
				// transfer selected call into selected conference
				callRequests.transferToConference(selectedConferenceId, selectedCallId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doCallNumberToConference()
	{
		
		if (!StringUtils.isEmpty(selectedConferenceId) && !StringUtils.isEmpty(destinationNumber))
		{
			try
			{
				// call number and pull into selected conference
				callRequests.callNumberToConference(selectedConferenceId, destinationNumber);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doJoinToConference()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// join the conference
				callRequests.joinToConference(selectedConferenceId, phoneToUseProvider.getPhoneToUse());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doLeaveConference()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// leave the selected conference
				callRequests.leaveConference(selectedConferenceId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doQuitConference()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// quit the selected conference
				callRequests.quitConference(selectedConferenceId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doMuteToConference()
	{
		if (selectedConferenceCall != null)
		{
			try
			{
				// switch this calls mute permission
				callRequests.muteToConference(selectedConferenceCall.getId(), !selectedConferenceCall.isMuted());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doListenToConference()
	{
		if (selectedConferenceCall != null)
		{
			try
			{
				// switch this calls listen permission
				callRequests.listenToConference(selectedConferenceCall.getId(),
					!selectedConferenceCall.isListening());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doWantToTalkToConference()
	{
		if (selectedConferenceCall != null)
		{
			try
			{
				// request talk permission
				callRequests.wantToTalkToConference(selectedConferenceCall.getId());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doAllCanHearInRoom()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// All participants can hear in the conference room
				callRequests.allHearInRoom(selectedConferenceId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doAllCanSpeakInRoom()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// All participants can speak in the conference room
				callRequests.allCanSpeakInRoom(selectedConferenceId);
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doOnlyOneCanSpeakInRoom()
	{
		if (selectedConferenceCall != null)
		{
			try
			{
				// only this call can speak in the conference room
				callRequests.onlyOneCanSpeakInRoom(selectedConferenceCall.getId());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doStartManagedConference()
	{
		if (!StringUtils.isEmpty(selectedConferenceId))
		{
			try
			{
				// Start the selected conference
				callRequests.startManagedConferenceById(selectedConferenceId, phoneToUseProvider.getPhoneToUse());
			}
			catch (UciException ex)
			{
				// Too simple error management. MUST be improved in real client!
				ex.printStackTrace();
			}
		}
	}
	
	private void doCreateUnmanagedConference()
	{
		try
		{
			// Start unmanaged conference
			callRequests.startManagedConferenceById("", phoneToUseProvider.getPhoneToUse());
		}
		catch (UciException ex)
		{
			// Too simple error management. MUST be improved in real client!
			ex.printStackTrace();
		}
	}
	
	public void removeAllCalls()
	{
		callTableModel.removeAllCalls();
	}
	
	public void removeCall(String callId)
	{
		callTableModel.removeCall(callId);
	}
	
	private void updateCall(Call call)
	{
		callTableModel.updateCall(call);
	}
	
	private void removeConferenceCall(ConferenceCall call)
	{
		if (call != null)
		{
			final String callId = call.getId();
			if (selectedConferenceCall != null)
			{
				if (StringUtils.equals(selectedConferenceCall.getId(), callId))
				{
					clearSelectedConferenceCallProperties();
				}
			}
			if (StringUtils.equals(selectedConferenceId, call.getRoomId()))
			{
				conferenceCallTableModel.removeConferenceCall(callId);
				if (conferenceCallTableModel.getRowCount() == 0)
				{
					// TODO the selected conference is empty; update the room list?
				}
			}
		}
	}
	
	private void updateConferenceCall(ConferenceCall call)
	{
		if (call != null && selectedConferenceCall != null
			&& StringUtils.equals(selectedConferenceCall.getId(), call.getId()))
		{
			// received an update for the currently selected conference call
			setSelectedConferenceCallProperties(call);
		}
		
		if (call != null)
		{
			final String roomId = call.getRoomId();
			if (!conferenceIds.contains(roomId))
			{
				List<String> tmp = new ArrayList<String>(conferenceIds);
				tmp.add(roomId);
				setConferenceIds(tmp);
			}
			// the received conference call belongs to the currently selected room, update it in the table
			if (StringUtils.equals(selectedConferenceId, roomId))
			{
				conferenceCallTableModel.updateConferenceCall(call);
			}
		}
		
	}
	
	private void doUpdateSelectedCall()
	{
		if (selectedCallTableRows.size() == 1)
		{
			Call call = callTableModel.getCall(selectedCallTableRows.get(0));
			setSelectedCallProperties(call);
		}
		else
		{
			clearSelectedCallProperties();
		}
	}
	
	private void clearSelectedCallProperties()
	{
		setSelectedCallId("");
	}
	
	private void setSelectedCallProperties(Call call)
	{
		setSelectedCallId(call.getId());
	}
	
	private void doUpdateSelectedConferenceCall()
	{
		if (selectedConferenceCallTableRows.size() == 1)
		{
			ConferenceCall call = conferenceCallTableModel.getConferenceCall(selectedConferenceCallTableRows.get(0));
			setSelectedConferenceCallProperties(call);
		}
		else
		{
			clearSelectedConferenceCallProperties();
		}
	}
	
	private void clearSelectedConferenceCallProperties()
	{
		setSelectedConferenceCall(null);
		setSelectedConferenceCallId("");
	}
	
	private void setSelectedConferenceCallProperties(ConferenceCall call)
	{
		setSelectedConferenceCall(call);
		setSelectedConferenceCallId(call.getId());
	}
	
	private final ListSelectionListener selectedCallTableRowsListSelectionListener = new ListSelectionListener()
	{
		public void valueChanged(ListSelectionEvent e)
		{
			//doCheckSelectedCalls();
			doUpdateSelectedCall();
		}
	};
	
	private final ListSelectionListener selectedConferenceCallTableRowsListSelectionListener = new ListSelectionListener()
	{
		public void valueChanged(ListSelectionEvent e)
		{
			
			doUpdateSelectedConferenceCall();
			//doCheckSelectedCalls();
		}
	};
	
	private ActionListener hangupCallActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doHangupCall();
		}
	};
	
	//	private ActionListener mediateToConferenceActionListener = new ActionListener()
	//	{
	//		@Override
	//		public void actionPerformed(ActionEvent e)
	//		{
	//			doMediateCallForConference();
	//		}
	//	};
	
	private ActionListener transferToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doTransferCallForConference();
		}
	};
	
	private ActionListener callNumberToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCallNumberToConference();
		}
	};
	private ActionListener quitConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doQuitConference();
		}
	};
	private ActionListener leaveConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doLeaveConference();
		}
	};
	private ActionListener joinToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doJoinToConference();
		}
	};
	private ActionListener listenToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doListenToConference();
		}
	};
	private ActionListener muteToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doMuteToConference();
		}
	};
	private ActionListener wantToTalkToConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doWantToTalkToConference();
		}
	};
	
	private ActionListener getAllConferencesActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetAllConferenceIds();
		}
	};
	
	private ActionListener getAllConferenceCallsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetAllConferenceCalls();
		}
	};
	
	private ActionListener allCanHearInRoomActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doAllCanHearInRoom();
		}
	};
	
	private ActionListener allCanSpeakInRoomActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doAllCanSpeakInRoom();
		}
	};
	
	private ActionListener onlyOneCanSpeakInRoomActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doOnlyOneCanSpeakInRoom();
		}
	};
	
	private ActionListener startManagedConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doStartManagedConference();
		}
	};
	
	private ActionListener createUnmanagedConferenceActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCreateUnmanagedConference();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private CallTableModel callTableModel = new CallTableModel();
	
	private ConferenceCallTableModel conferenceCallTableModel = new ConferenceCallTableModel();
	
	private List<Integer> selectedCallTableRows = new ArrayList<Integer>();
	
	private List<Integer> selectedConferenceCallTableRows = new ArrayList<Integer>();
	
	private List<String> conferenceIds = new ArrayList<String>();
	
	private ConferenceCall selectedConferenceCall = null;
	
	private String selectedConferenceId = "";
	
	private String selectedConferenceCallId = "";
	
	private String selectedCallId = "";
	
	private String destinationNumber = Config.getValue("destinationNumber");
	
	private String originatingNumber = Config.getValue("originatingNumber");
	
	private String newCallId = Config.getValue("callId");
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public TableModel getCallTableModel()
	{
		return callTableModel;
	}
	
	public TableModel getConferenceCallTableModel()
	{
		return conferenceCallTableModel;
	}
	
	public List<Integer> getSelectedCallTableRows()
	{
		return selectedCallTableRows;
	}
	
	public List<Integer> getSelectedConferenceCallTableRows()
	{
		return selectedConferenceCallTableRows;
	}
	
	public static final String PROPERTY_CONFERENCE_IDS = "conferenceIds";
	
	public List<String> getConferenceIds()
	{
		return conferenceIds;
	}
	
	public void setConferenceIds(List<String> conferenceIds)
	{
		List<String> old = this.conferenceIds;
		this.conferenceIds = conferenceIds;
		changeSupport.firePropertyChange(PROPERTY_CONFERENCE_IDS, old, conferenceIds);
		if (this.conferenceIds.size() == 0)
		{
			conferenceCallTableModel.setConferenceCalls(new ArrayList<ConferenceCall>());
			clearSelectedConferenceCallProperties();
		}
	}
	
	public static final String PROPERTY_DESTINATION_NUMBER = "destinationNumber";
	
	public String getDestinationNumber()
	{
		return destinationNumber;
	}
	
	public void setDestinationNumber(String destinationNumber)
	{
		String old = this.destinationNumber;
		this.destinationNumber = destinationNumber;
		changeSupport.firePropertyChange(PROPERTY_DESTINATION_NUMBER, old, destinationNumber);
	}
	
	public String getOriginatingNumber()
	{
		return originatingNumber;
	}
	
	public static final String PROPERTY_ORIGINATING_NUMBER = "originatingNumber";
	
	public void setOriginatingNumber(String originatingNumber)
	{
		String old = this.originatingNumber;
		this.originatingNumber = originatingNumber;
		changeSupport.firePropertyChange(PROPERTY_ORIGINATING_NUMBER, old, originatingNumber);
	}
	
	public String getNewCallId()
	{
		return newCallId;
	}
	
	public static final String PROPERTY_NEW_CONFERENCECALL_ID = "newCallId";
	
	public void setNewCallId(String callId)
	{
		String old = this.newCallId;
		this.newCallId = callId;
		changeSupport.firePropertyChange(PROPERTY_NEW_CONFERENCECALL_ID, old, callId);
	}
	
	public static final String PROPERTY_SELECTED_CONFERENCE_ID = "selectedConferenceId";
	
	public String getSelectedConferenceId()
	{
		return selectedConferenceId;
	}
	
	public void setSelectedConferenceId(String selectedConferenceId)
	{
		String old = this.selectedConferenceId;
		this.selectedConferenceId = selectedConferenceId;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_CONFERENCE_ID, old, selectedConferenceId);
	}
	
	public static final String PROPERTY_SELECTED_CONFERENCECALL = "selectedConferenceCall";
	
	public ConferenceCall getSelectedConferenceCall()
	{
		return selectedConferenceCall;
	}
	
	public void setSelectedConferenceCall(ConferenceCall selectedConferenceCall)
	{
		ConferenceCall old = this.selectedConferenceCall;
		this.selectedConferenceCall = selectedConferenceCall;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_CONFERENCECALL, old, selectedConferenceCall);
	}
	
	public String getSelectedConferenceCallId()
	{
		return selectedConferenceCallId;
	}
	
	public void setSelectedConferenceCallId(String selectedConferenceCallId)
	{
		this.selectedConferenceCallId = selectedConferenceCallId;
	}
	
	public String getSelectedCallId()
	{
		return selectedCallId;
	}
	
	public void setSelectedCallId(String selectedCallId)
	{
		this.selectedCallId = selectedCallId;
	}
	
	public ActionListener getHangupCallActionListener()
	{
		return hangupCallActionListener;
	}
	
	//	public ActionListener getMediateToConferenceActionListener()
	//	{
	//		return mediateToConferenceActionListener;
	//	}
	
	public ActionListener getTransferToConferenceActionListener()
	{
		return transferToConferenceActionListener;
	}
	
	public ActionListener getCallNumberToConferenceActionListener()
	{
		return callNumberToConferenceActionListener;
	}
	
	public ActionListener getJoinToConferenceActionListener()
	{
		return joinToConferenceActionListener;
	}
	
	public ActionListener getListentoConferenceActionListener()
	{
		return listenToConferenceActionListener;
	}
	
	public ActionListener getMutetoConferenceActionListener()
	{
		return muteToConferenceActionListener;
	}
	
	public ActionListener getWantToTalktoConferenceActionListener()
	{
		return wantToTalkToConferenceActionListener;
	}
	
	public ActionListener getQuitConferenceActionListener()
	{
		return quitConferenceActionListener;
	}
	
	public ActionListener getLeaveConferenceActionListener()
	{
		return leaveConferenceActionListener;
	}
	
	public ActionListener getGetAllConferenceActionListener()
	{
		return getAllConferencesActionListener;
	}
	
	public ActionListener getGetAllConferenceCallsActionListener()
	{
		return getAllConferenceCallsActionListener;
	}
	
	/**
	 * @return the allCanHearInRoomActionListener
	 */
	public ActionListener getAllCanHearInRoomActionListener()
	{
		return allCanHearInRoomActionListener;
	}
	
	/**
	 * @return the allCanSpeakInRoomActionListener
	 */
	public ActionListener getAllCanSpeakInRoomActionListener()
	{
		return allCanSpeakInRoomActionListener;
	}
	
	/**
	 * @return the onlyOneCanSpeakInRoomActionListener
	 */
	public ActionListener getOnlyOneCanSpeakInRoomActionListener()
	{
		return onlyOneCanSpeakInRoomActionListener;
	}
	
	/**
	 * @return the starManagedConferenceActionListener
	 */
	public ActionListener getStartManagedConferenceActionListener()
	{
		return startManagedConferenceActionListener;
	}
	
	public ActionListener getCreateUnmanagedConferenceActionListener()
	{
		return createUnmanagedConferenceActionListener;
	}
	
	public ListSelectionListener getSelectedCallTableRowsListener()
	{
		return selectedCallTableRowsListSelectionListener;
	}
	
	public ListSelectionListener getSelectedConferenceCallTableRowsListener()
	{
		return selectedConferenceCallTableRowsListSelectionListener;
	}
}
