package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.List;

import javax.swing.table.TableModel;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciGroupEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciGroupRequests;
import de.starface.integration.uci.java.v30.types.GroupSetting;
import de.starface.integration.uci.v30.client.UciServiceRequests;
import de.starface.integration.uci.v30.client.utils.EventsProvider;
import de.starface.integration.uci.v30.client.utils.EventsProviderSupport;

public class GroupViewModel implements UciGroupEvents, EventsProvider<UciGroupEvents>
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private EventsProviderSupport<UciGroupEvents> eventsSupport = new EventsProviderSupport<UciGroupEvents>(
		UciGroupEvents.class);
	
	private UciGroupRequests groupRequests;
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.groupRequests = uciServiceRequests.getRequests(UciGroupRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciGroupEvents.class);
	}
	
	@Override
	public void groupSettingsChanged(List<GroupSetting> groupSettings)
	{
		groupSettingsTableModel.setGroupSettings(groupSettings);
		eventsSupport.getEventsDistributor().groupSettingsChanged(groupSettings);
	}
	
	@Override
	public void subscribeEvents(UciGroupEvents listener)
	{
		eventsSupport.subscribeEvents(listener);
	}
	
	@Override
	public void unsubscribeEvents(UciGroupEvents listener)
	{
		eventsSupport.unsubscribeEvents(listener);
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void doGetGroupSettings()
	{
		try
		{
			// Retrieve the group settings sending the getGroupSettings request
			List<GroupSetting> groupSettings = groupRequests.getGroupSettings();
			groupSettingsTableModel.setGroupSettings(groupSettings);
			eventsSupport.getEventsDistributor().groupSettingsChanged(groupSettings);
		}
		catch (UciException e)
		{
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}
	
	private void doSetGroupSetting()
	{
		if (selectedRow >= 0)
		{
			GroupSetting groupSetting = groupSettingsTableModel.getGroupSetting(selectedRow);
			try
			{
				groupRequests.setGroupSetting(groupSetting);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doSetGroupSettings()
	{
		try
		{
			groupRequests.setGroupSettings(groupSettingsTableModel.getGroupSettings());
		}
		catch (UciException e)
		{
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}
	
	private final ActionListener getGroupSettingsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetGroupSettings();
		}
	};
	
	private final ActionListener setGroupSettingActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doSetGroupSetting();
		}
	};
	
	private final ActionListener setGroupSettingsActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doSetGroupSettings();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private final GroupSettingsTableModel groupSettingsTableModel = new GroupSettingsTableModel();
	
	private int selectedRow = -1;
	
	private PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public int getSelectedRow()
	{
		return selectedRow;
	}
	
	public static final String PROPERTY_SELECTED_ROW = "selectedRow";
	
	public void setSelectedRow(int selectedRow)
	{
		int old = this.selectedRow;
		this.selectedRow = selectedRow;
		changeSupport.firePropertyChange(PROPERTY_SELECTED_ROW, old, selectedRow);
	}
	
	public TableModel getGroupSettingsTableModel()
	{
		return groupSettingsTableModel;
	}
	
	public ActionListener getSetGroupSettingActionListener()
	{
		return setGroupSettingActionListener;
	}
	
	public ActionListener getSetGroupSettingsActionListener()
	{
		return setGroupSettingsActionListener;
	}
	
	public ActionListener getGetGroupSettingsActionListener()
	{
		return getGroupSettingsActionListener;
	}
	
}
