package de.starface.examples.uci30.swing.viewmodel;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import de.starface.examples.uci30.swing.model.Config;
import de.starface.examples.uci30.swing.model.ConnectionManager;

public class LoginViewModel
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private final ConnectionManager connectionManager;
	
	public LoginViewModel(ConnectionManager connectionManager)
	{
		this.connectionManager = connectionManager;
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void doConnect()
	{
		connectionManager.connect(useXmpp, xmppHost, httpHost, httpCallBackHost, loginId, password);
	}
	
	private final ActionListener connectActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doConnect();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private String httpHost = Config.getValue("httpHost");
	
	private String xmppHost = Config.getValue("xmppHost");
	
	private String loginId = Config.getValue("loginId");
	
	private String password = Config.getValue("password");
	
	private String httpCallBackHost = Config.getValue("httpCallBackHost");
	
	private boolean useXmpp = true;
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener x)
	{
		changeSupport.addPropertyChangeListener(x);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener x)
	{
		changeSupport.removePropertyChangeListener(x);
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public String getHttpHost()
	{
		return httpHost;
	}
	
	public static final String PROPERTY_HTTP_HOST = "httpHost";
	
	public void setHttpHost(String httpHost)
	{
		String old = this.httpHost;
		this.httpHost = httpHost;
		changeSupport.firePropertyChange(PROPERTY_HTTP_HOST, old, httpHost);
	}
	
	public String getXmppHost()
	{
		return xmppHost;
	}
	
	public static final String PROPERTY_XMPP_HOST = "xmppHost";
	
	public void setXmppHost(String xmppHost)
	{
		String old = this.xmppHost;
		this.xmppHost = xmppHost;
		changeSupport.firePropertyChange(PROPERTY_XMPP_HOST, old, xmppHost);
	}
	
	public String getLoginId()
	{
		return loginId;
	}
	
	public static final String PROPERTY_LOGIN_ID = "loginId";
	
	public void setLoginId(String loginId)
	{
		String old = this.loginId;
		this.loginId = loginId;
		changeSupport.firePropertyChange(PROPERTY_LOGIN_ID, old, loginId);
	}
	
	public String getPassword()
	{
		return password;
	}
	
	public static final String PROPERTY_PASSWORD = "password";
	
	public void setPassword(String password)
	{
		String old = this.password;
		this.password = password;
		changeSupport.firePropertyChange(PROPERTY_PASSWORD, old, password);
	}
	
	public String getHttpCallBackHost()
	{
		return httpCallBackHost;
	}
	
	public static final String PROPERTY_HTTP_CALL_BACK_HOST = "httpCallBackHost";
	
	public void setHttpCallBackHost(String httpCallBackHost)
	{
		String old = this.httpCallBackHost;
		this.httpCallBackHost = httpCallBackHost;
		changeSupport.firePropertyChange(PROPERTY_HTTP_CALL_BACK_HOST, old, httpCallBackHost);
	}
	
	public boolean isUseXmpp()
	{
		return useXmpp;
	}
	
	public static final String PROPERTY_USE_XMPP = "useXmpp";
	
	public void setUseXmpp(boolean useXmpp)
	{
		boolean old = this.useXmpp;
		this.useXmpp = useXmpp;
		changeSupport.firePropertyChange(PROPERTY_USE_XMPP, old, useXmpp);
	}
	
	public ActionListener getConnectActionListener()
	{
		return connectActionListener;
	}
	
}
