package de.starface.examples.uci30.swing.viewmodel;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import org.apache.commons.lang.StringUtils;

import de.starface.integration.uci.java.v30.types.MonitorListEntry;
import de.starface.integration.uci.java.v30.values.MonitorListEntryProperties;

public class MonitorListTableModel implements TableModel
{
	
	private static final MonitorListEntryProperties[] columnOrder = new MonitorListEntryProperties[MonitorListEntryProperties.values().length];
	
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>();
	
	private List<MonitorListEntry> listEntries;
	
	static
	{
		columnOrder[0] = MonitorListEntryProperties.id;
		columnOrder[1] = MonitorListEntryProperties.callercallerid;
		columnOrder[2] = MonitorListEntryProperties.calledcallerid;
		columnOrder[3] = MonitorListEntryProperties.callbacknumber;
		columnOrder[4] = MonitorListEntryProperties.file;
		columnOrder[5] = MonitorListEntryProperties.monitortimestart;
		columnOrder[6] = MonitorListEntryProperties.monitortimeend;
		columnOrder[7] = MonitorListEntryProperties.duration;
	}
	
	public MonitorListTableModel()
	{
		listEntries = new ArrayList<MonitorListEntry>();
	}
	
	@Override
	public int getRowCount()
	{
		return listEntries.size();
	}
	
	@Override
	public int getColumnCount()
	{
		return columnOrder.length;
	}
	
	@Override
	public String getColumnName(int columnIndex)
	{
		return columnOrder[columnIndex].name();
	}
	
	@Override
	public Class<?> getColumnClass(int columnIndex)
	{
		return columnOrder[columnIndex].getType();
	}
	
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		// TODO Auto-generated method stub
		return false;
	}
	
	@Override
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		if (rowIndex > listEntries.size() - 1)
		{
			return null;
		}
		else
		{
			return Utils.getProperty(columnOrder[columnIndex], listEntries.get(rowIndex));
		}
	}
	
	@Override
	public void setValueAt(Object aValue, int rowIndex, int columnIndex)
	{
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public void addTableModelListener(TableModelListener listener)
	{
		listeners.add(listener);
		
	}
	
	@Override
	public void removeTableModelListener(TableModelListener listener)
	{
		listeners.remove(listener);
	}
	
	private void fireTableChanged(TableModelEvent event)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(event);
			}
		}
	}
	
	public void setMonitorList(List<MonitorListEntry> listEntries)
	{
		this.listEntries = listEntries;
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void removeEntry(String entryId)
	{
		int rowIndex = 0;
		for (ListIterator<MonitorListEntry> listIterator = listEntries.listIterator(); listIterator.hasNext(); rowIndex = listIterator.nextIndex())
		{
			if (StringUtils.equals(listIterator.next().getId(), entryId))
			{
				listIterator.remove();
				fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, TableModelEvent.ALL_COLUMNS,
					TableModelEvent.DELETE));
			}
		}
	}
	
	// iterate over all listEntries; for every entry, check if it needs to be removed
	public void removeEntries(List<String> entryIds)
	{
		boolean removedEntry = false;
		for (ListIterator<MonitorListEntry> listIterator = listEntries.listIterator(); listIterator.hasNext();)
		{
			if (entryIds.contains(listIterator.next().getId()))
			{
				listIterator.remove();
				removedEntry = true;
			}
		}
		if (removedEntry)
		{
			fireTableChanged(new TableModelEvent(this));
		}
	}
	
	public MonitorListEntry getListEntry(int rowIndex)
	{
		if (rowIndex > listEntries.size() - 1)
		{
			return null;
		}
		else
		{
			return listEntries.get(rowIndex);
		}
	}
}
