/*
 * $Id: RedirectSettingsTableModel.java 515 2011-03-11 19:14:15Z tilmann $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2011-03-11 20:14:15 +0100 (Fr, 11 Mrz 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 515 $
 */
package de.starface.examples.uci30.swing.viewmodel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.starface.integration.uci.java.v30.types.QueueSetting;
import de.starface.integration.uci.java.v30.values.QueueSettingProperties;

/**
 * A swing table model for the redirect settings
 * 
 * @author Mojo
 */
public class QueueSettingsTableModel implements TableModel
{
	private static final QueueSettingProperties[] queueColumnOrder;
	
	static
	{
		queueColumnOrder = new QueueSettingProperties[5];
		queueColumnOrder[0] = QueueSettingProperties.name;
		queueColumnOrder[1] = QueueSettingProperties.id;
		queueColumnOrder[2] = QueueSettingProperties.accountId;
		queueColumnOrder[3] = QueueSettingProperties.maxWaitingTime;
		queueColumnOrder[4] = QueueSettingProperties.size;
	}
	
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>(1);
	
	private List<QueueSetting> queueSettings = new ArrayList<QueueSetting>();
	
	public void addTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.add(l);
		}
	}
	
	public Class<?> getColumnClass(int columnIndex)
	{
		switch (columnIndex)
		{
			case 2:
				return String.class;
			default:
				return queueColumnOrder[columnIndex].getType();
		}
	}
	
	public int getColumnCount()
	{
		return queueColumnOrder.length;
	}
	
	public String getColumnName(int columnIndex)
	{
		return queueColumnOrder[columnIndex].name();
	}
	
	public int getRowCount()
	{
		return queueSettings.size();
	}
	
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			default:
				return Utils.getProperty(queueColumnOrder[columnIndex], queueSettings.get(rowIndex));
		}
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			case 3:
				return true;
			default:
				return false;
		}
	}
	
	public List<QueueSetting> getQueueSettings()
	{
		return queueSettings;
	}
	
	public void setQueueSettings(List<QueueSetting> QueueSettings)
	{
		this.queueSettings = QueueSettings;
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void updateQueueSetting(Integer queueAccountId, QueueSetting queueSetting)
	{
		Iterator<QueueSetting> it = queueSettings.iterator();
		while (it.hasNext())
		{
			QueueSetting oldSetting = it.next();
			if (oldSetting.getAccountId() == queueAccountId)
			{
				it.remove();
				break;
			}
		}
		if (queueSetting != null)
		{
			this.queueSettings.add(queueSetting);
		}
		fireTableChanged(new TableModelEvent(this));
	}
	
	public QueueSetting getQueueSetting(int rowIndex)
	{
		return queueSettings.get(rowIndex);
	}
	
	public void removeTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.remove(l);
		}
	}
	
	public void setValueAt(Object value, int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			case 3:
				Utils.setProperty(queueColumnOrder[columnIndex], queueSettings.get(rowIndex), value);
			default:
		}
	}
	
	private void fireTableChanged(TableModelEvent e)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(e);
			}
		}
	}
	
	public String getQueueName(int accountId)
	{
		for (QueueSetting qs : queueSettings)
		{
			if (qs.getAccountId() == accountId)
			{
				return qs.getName();
			}
		}
		return null;
	}
	
}
