/*
 * $Id: RedirectSettingsTableModel.java 851 2013-11-20 11:23:28Z dirk $
 * 
 * Copyright (C) by vertico software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * $LastChangedDate: 2013-11-20 12:23:28 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 851 $
 */
package de.starface.examples.uci30.swing.viewmodel;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import de.starface.integration.uci.java.v30.types.RedirectSetting;
import de.starface.integration.uci.java.v30.values.RedirectSettingDestinationType;
import de.starface.integration.uci.java.v30.values.RedirectSettingProperties;
import de.starface.integration.uci.java.v30.values.RedirectSettingType;

/**
 * A swing table model for the redirect settings
 * 
 * @author tilmann
 */
public class RedirectSettingsTableModel implements TableModel
{
	private static final RedirectSettingProperties[] columnOrder = new RedirectSettingProperties[RedirectSettingProperties.values().length - 3];
	
	static
	{
		columnOrder[0] = RedirectSettingProperties.type;
		columnOrder[1] = RedirectSettingProperties.enabled;
		columnOrder[2] = RedirectSettingProperties.groupId;
		columnOrder[3] = RedirectSettingProperties.calledNumber;
		columnOrder[4] = RedirectSettingProperties.timeout;
		columnOrder[5] = RedirectSettingProperties.destinationType;
		columnOrder[6] = RedirectSettingProperties.destination;
	}
	
	private List<TableModelListener> listeners = new ArrayList<TableModelListener>(1);
	
	private List<RedirectSetting> redirectSettings = new ArrayList<RedirectSetting>();
	
	public void setRedirectSettings(List<RedirectSetting> redirectSettings)
	{
		this.redirectSettings = redirectSettings;
		fireTableChanged(new TableModelEvent(this));
	}
	
	public void addTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.add(l);
		}
	}
	
	public Class<?> getColumnClass(int columnIndex)
	{
		return columnOrder[columnIndex].getType();
	}
	
	public int getColumnCount()
	{
		return columnOrder.length;
	}
	
	public String getColumnName(int columnIndex)
	{
		return columnOrder[columnIndex].name();
	}
	
	public int getRowCount()
	{
		return redirectSettings.size();
	}
	
	public Object getValueAt(int rowIndex, int columnIndex)
	{
		return Utils.getProperty(columnOrder[columnIndex], redirectSettings.get(rowIndex));
	}
	
	public boolean isCellEditable(int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			case 1:
			case 4:
			case 5:
			case 6:
				return true;
			default:
				return false;
		}
	}
	
	public List<RedirectSetting> getRedirectSettings()
	{
		return redirectSettings;
	}
	
	public RedirectSetting getRedirectSetting(int rowIndex)
	{
		return redirectSettings.get(rowIndex);
	}
	
	public void removeTableModelListener(TableModelListener l)
	{
		synchronized (listeners)
		{
			listeners.remove(l);
		}
	}
	
	public void setValueAt(Object value, int rowIndex, int columnIndex)
	{
		switch (columnIndex)
		{
			case 1:
			case 4:
			case 5:
			case 6:
				Utils.setProperty(columnOrder[columnIndex], redirectSettings.get(rowIndex), value);
			default:
		}
		if (columnIndex == 5)
		{
			RedirectSetting redirectSetting = redirectSettings.get(rowIndex);
			if (redirectSetting.getDestinationType() == RedirectSettingDestinationType.VOICEMAIL)
			{
				redirectSetting.setDestination(redirectSetting.getLastDestinationMailbox());
			}
			else
			{
				redirectSetting.setDestination(redirectSetting.getLastDestinationNumber());
			}
			fireTableChanged(new TableModelEvent(this, rowIndex, rowIndex, 6, TableModelEvent.UPDATE));
		}
	}
	
	private void fireTableChanged(TableModelEvent e)
	{
		synchronized (listeners)
		{
			for (TableModelListener l : listeners)
			{
				l.tableChanged(e);
			}
		}
	}
	
	public void updateRedirectSettings(List<RedirectSetting> updatedRedirectSettings)
	{
		for (RedirectSetting updatedSetting : updatedRedirectSettings)
		{
			int row = getRedirectSettingRow(redirectSettings, updatedSetting.getCalledNumber(),
				updatedSetting.getType());
			if (row < 0)
			{
				redirectSettings.add(updatedSetting);
			}
			else
			{
				redirectSettings.set(row, updatedSetting);
			}
		}
		for (Iterator<RedirectSetting> iter = redirectSettings.iterator(); iter.hasNext();)
		{
			RedirectSetting redirectSetting = iter.next();
			int row = getRedirectSettingRow(updatedRedirectSettings, redirectSetting.getCalledNumber(),
				redirectSetting.getType());
			if (row < 0)
			{
				iter.remove();
			}
		}
		fireTableChanged(new TableModelEvent(this));
	}
	
	private static int getRedirectSettingRow(List<RedirectSetting> settings, String calledNumber,
											 RedirectSettingType type)
	{
		for (int row = 0; row < settings.size(); row++)
		{
			RedirectSetting setting = settings.get(row);
			if (type == setting.getType() && calledNumber.equals(setting.getCalledNumber()))
			{
				return row;
			}
		}
		return -1;
	}
	
}
