package de.starface.examples.uci30.swing.viewmodel;

import java.awt.Desktop;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.swing.table.TableModel;

import org.apache.commons.lang.StringUtils;

import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.messages.events.UciVoicemailListEvents;
import de.starface.integration.uci.java.v30.messages.requests.UciVoicemailListRequests;
import de.starface.integration.uci.java.v30.types.Mailbox;
import de.starface.integration.uci.java.v30.types.VoicemailList;
import de.starface.integration.uci.java.v30.types.VoicemailListEntry;
import de.starface.integration.uci.java.v30.values.OrderDirection;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryFolder;
import de.starface.integration.uci.java.v30.values.VoicemailListEntryProperties;
import de.starface.integration.uci.v30.client.UciServiceRequests;

public class VoicemailListViewModel implements UciVoicemailListEvents
{
	/// Domain Logic Access
	////////////////////////////////////////////////////////////////
	
	private UciVoicemailListRequests voicemailListRequests;
	
	private CallViewModel callViewModel;
	
	private PhoneToUseProvider phoneToUseProvider;
	
	public VoicemailListViewModel(PhoneToUseProvider phoneToUseProvider, CallViewModel callViewModel)
	{
		this.phoneToUseProvider = phoneToUseProvider;
		this.callViewModel = callViewModel;
	}
	
	public void setUciServiceRequests(UciServiceRequests uciServiceRequests) throws UciException
	{
		this.voicemailListRequests = uciServiceRequests.getRequests(UciVoicemailListRequests.class);
		uciServiceRequests.subscribeEventsAsync(this, UciVoicemailListEvents.class);
	}
	
	@Override
	public void newVoicemailListEntry(VoicemailListEntry voicemailListEntry)
	{
		voicemailListTableModel.addEntry(voicemailListEntry);
	}
	
	/// View Actions
	////////////////////////////////////////////////////////////////
	
	private void doBrowseVoicemailFileUrl()
	{
		if (!StringUtils.isEmpty(voicemailFileUrl))
		{
			Desktop desktop = Desktop.getDesktop();
			try
			{
				desktop.browse(new URL(voicemailFileUrl).toURI());
			}
			catch (Exception e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doOpenVoicemailFile()
	{
		if (!StringUtils.isEmpty(voicemailFile))
		{
			Desktop desktop = Desktop.getDesktop();
			try
			{
				desktop.open(new File(voicemailFile));
			}
			catch (Exception e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doGetVoicemailList()
	{
		if (voicemailListRequests != null)
		{
			try
			{
				Date receivedBefore = this.startedBefore != null ? this.startedBefore : new Date();
				Date receivedAfter = this.startedAfter != null ? this.startedAfter : new Date(0);
				String mailboxRestrictionId = mailboxRestriction != null ? mailboxRestriction.getId() : "";
				
				// Retrieve call list sending getCallList request
				VoicemailList voicemailList = voicemailListRequests.getVoicemailList(receivedBefore,
					receivedAfter, mailboxRestrictionId, folderRestriction, orderProperty, orderDirection,
					countOffset, countLimit);
				
				voicemailListTableModel.setVoicemailList(voicemailList);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doDeleteVoicemailListEntry()
	{
		if (isEntrySelected())
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				// delete call list entry sending deleteCallListEntry request
				voicemailListRequests.deleteVoicemailListEntry(entry.getId());
				
				voicemailListTableModel.removeEntry(selectedRows.get(0));
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doDeleteVoicemailListEntries()
	{
		if (!selectedRows.isEmpty() && voicemailListRequests != null)
		{
			List<String> idsToDelete = computeSelectedEntryIds();
			try
			{
				voicemailListRequests.deleteVoicemailListEntries(idsToDelete);
				voicemailListTableModel.removeEntries(selectedRows);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private List<String> computeSelectedEntryIds()
	{
		List<String> entryIds = new ArrayList<String>(selectedRows.size());
		for (Integer i : selectedRows)
		{
			entryIds.add(voicemailListTableModel.getEntry(i).getId());
		}
		return entryIds;
	}
	
	private void doMoveVoicemailListEntry()
	{
		if (isEntrySelected())
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				// Move fax list entry sending moveFaxListEntry request
				voicemailListRequests.moveVoicemailListEntry(entry.getId(), destinationFolder);
				
				if (destinationFolder != folderRestriction && folderRestriction != null)
				{
					voicemailListTableModel.removeEntry(selectedRows.get(0));
				}
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doMoveVoicemailListEntries()
	{
		if (!selectedRows.isEmpty() && voicemailListRequests != null)
		{
			List<String> idsToMove = computeSelectedEntryIds();
			try
			{
				voicemailListRequests.moveVoicemailListEntries(idsToMove, destinationFolder);
				
				if (destinationFolder != folderRestriction && folderRestriction != null)
				{
					voicemailListTableModel.removeEntries(selectedRows);
				}
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doGetVoicemailFileUrl()
	{
		if (isEntrySelected())
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				final String fileUrl = voicemailListRequests.getVoicemailFileUrl(entry.getId());
				setVoicemailFileUrl(fileUrl);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doGetVoicemailFile()
	{
		if (isEntrySelected())
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				final File file = voicemailListRequests.getVoicemailFile(entry.getId());
				setVoicemailFile(file.getAbsolutePath());
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private boolean isEntrySelected()
	{
		return selectedRows.size() == 1;
	}
	
	private VoicemailListEntry getSelectedEntry()
	{
		if (isEntrySelected())
		{
			return voicemailListTableModel.getEntry(selectedRows.get(0));
		}
		else
		{
			return null;
		}
	}
	
	private void doGetMailboxes()
	{
		try
		{
			// Retrive mailboxes list sending getMailboxes request
			setMailboxValues(voicemailListRequests.getMailboxes());
		}
		catch (UciException e)
		{
			// Too simple error management. MUST be improved in real client!
			e.printStackTrace();
		}
	}
	
	private void doCallMailboxWithPhone()
	{
		if (selectedMailbox != null)
		{
			try
			{
				// Call the mailbox sending callMailboxWithPhone request
				voicemailListRequests.callMailboxWithPhone(selectedMailbox.getId(),
					phoneToUseProvider.getPhoneToUse());
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doCallMailboxWithNumber()
	{
		if (selectedMailbox != null && !StringUtils.isEmpty(originatingNumber))
		{
			try
			{
				// Call the mailbox sending callMailboxWithNumber request
				voicemailListRequests.callMailboxWithNumber(selectedMailbox.getId(), originatingNumber);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doCallVoicemailWithNumber()
	{
		if (isEntrySelected() && !StringUtils.isEmpty(originatingNumber))
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				// Listen to a voicemail sending callVoicemailWithNumber request
				voicemailListRequests.callVoicemailWithNumber(entry.getId(), originatingNumber);
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private void doCallVoicemailWithPhone()
	{
		if (isEntrySelected())
		{
			VoicemailListEntry entry = getSelectedEntry();
			try
			{
				// Listen to a voicemail sending callVoicemailWithPhone request
				voicemailListRequests.callVoicemailWithPhone(entry.getId(), phoneToUseProvider.getPhoneToUse());
			}
			catch (UciException e)
			{
				// Too simple error management. MUST be improved in real client!
				e.printStackTrace();
			}
		}
	}
	
	private final ActionListener callMailboxWithPhoneActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCallMailboxWithPhone();
		}
	};
	
	private final ActionListener callMailboxWithNumberActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCallMailboxWithNumber();
		}
	};
	
	private final ActionListener callVoicemailWithNumberActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCallVoicemailWithNumber();
		}
	};
	
	private final ActionListener callVoicemailWithPhoneActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doCallVoicemailWithPhone();
		}
	};
	
	private final ActionListener getGetMailboxesActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetMailboxes();
		}
	};
	
	private final ActionListener getVoicemailListActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetVoicemailList();
		}
	};
	
	private final ActionListener deleteVoicemailListEntryActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doDeleteVoicemailListEntry();
		}
	};
	
	private final ActionListener deleteVoicemailListEntriesActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doDeleteVoicemailListEntries();
		}
	};
	
	private final ActionListener getVoicemailFileUrlActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetVoicemailFileUrl();
		}
	};
	
	private final ActionListener getVoicemailFileActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doGetVoicemailFile();
		}
	};
	
	private final ActionListener moveVoicemailListEntryActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doMoveVoicemailListEntry();
		}
	};
	
	private final ActionListener moveVoicemailListEntriesActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doMoveVoicemailListEntries();
		}
	};
	
	private final ActionListener browseVoicemailFileUrlActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doBrowseVoicemailFileUrl();
		}
	};
	
	private final ActionListener openVoicemailFileActionListener = new ActionListener()
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			doOpenVoicemailFile();
		}
	};
	
	/// View Data
	////////////////////////////////////////////////////////////////
	
	private final VoicemailListTableModel voicemailListTableModel = new VoicemailListTableModel();
	
	private Date startedBefore = null;
	
	private Date startedAfter = null;
	
	private VoicemailListEntryFolder folderRestriction = null;
	
	private VoicemailListEntryProperties orderProperty = null;
	
	private OrderDirection orderDirection = null;
	
	private Mailbox mailboxRestriction = null;
	
	private int countOffset = 0;
	
	private int countLimit = 15;
	
	private List<Integer> selectedRows = new ArrayList<Integer>();
	
	private String voicemailFileUrl;
	
	private String voicemailFile;
	
	private List<Mailbox> mailboxValues;
	{
		setMailboxValues(Collections.<Mailbox> emptyList());
	}
	
	private List<Mailbox> mailboxRestrictionValues;
	
	private Mailbox selectedMailbox = null;
	
	private VoicemailListEntryFolder destinationFolder = VoicemailListEntryFolder.Old;
	
	private String originatingNumber = "";
	
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(this);
	
	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}
	
	/// Property Getters and Setters
	////////////////////////////////////////////////////////////////
	
	public TableModel getVoicemailListTableModel()
	{
		return voicemailListTableModel;
	}
	
	public String getVoicemailFile()
	{
		return voicemailFile;
	}
	
	public static final String PROPERTY_VOICEMAIL_FILE = "voicemailFile";
	
	public void setVoicemailFile(String voicemailFile)
	{
		String old = this.voicemailFile;
		this.voicemailFile = voicemailFile;
		changeSupport.firePropertyChange(PROPERTY_VOICEMAIL_FILE, old, voicemailFile);
	}
	
	public String getVoicemailFileUrl()
	{
		return voicemailFileUrl;
	}
	
	public static final String PROPERTY_VOICEMAIL_FILE_URL = "voicemailFileUrl";
	
	public void setVoicemailFileUrl(String faxFileUrl)
	{
		String old = this.voicemailFileUrl;
		this.voicemailFileUrl = faxFileUrl;
		changeSupport.firePropertyChange(PROPERTY_VOICEMAIL_FILE_URL, old, faxFileUrl);
	}
	
	public Date getStartedBefore()
	{
		return startedBefore;
	}
	
	public static final String PROPERTY_STARTED_BEFORE = "startedBefore";
	
	public void setStartedBefore(Date receivedBefore)
	{
		Date old = this.startedBefore;
		this.startedBefore = receivedBefore;
		changeSupport.firePropertyChange(PROPERTY_STARTED_BEFORE, old, receivedBefore);
	}
	
	public Date getStartedAfter()
	{
		return startedAfter;
	}
	
	public static final String PROPERTY_STARTED_AFTER = "startedAfter";;
	
	public void setStartedAfter(Date receivedAfter)
	{
		Date old = this.startedAfter;
		this.startedAfter = receivedAfter;
		changeSupport.firePropertyChange(PROPERTY_STARTED_AFTER, old, receivedAfter);
	}
	
	public VoicemailListEntryFolder getFolderRestriction()
	{
		return folderRestriction;
	}
	
	public static final String PROPERTY_FOLDER_RESTRICTION = "folderRestriction";
	
	public void setFolderRestriction(VoicemailListEntryFolder folderRestriction)
	{
		VoicemailListEntryFolder old = this.folderRestriction;
		this.folderRestriction = folderRestriction;
		changeSupport.firePropertyChange(PROPERTY_FOLDER_RESTRICTION, old, folderRestriction);
	}
	
	public VoicemailListEntryFolder[] getFolderRestrictionValues()
	{
		return Utils.createArrayWithLeadingNull(VoicemailListEntryFolder.values());
	}
	
	public VoicemailListEntryProperties getOrderProperty()
	{
		return orderProperty;
	}
	
	public static final String PROPERTY_ORDER_PROPERTY = "orderProperty";
	
	public void setOrderProperty(VoicemailListEntryProperties orderProperty)
	{
		VoicemailListEntryProperties old = this.orderProperty;
		this.orderProperty = orderProperty;
		changeSupport.firePropertyChange(PROPERTY_ORDER_PROPERTY, old, orderProperty);
	}
	
	public VoicemailListEntryProperties[] getOrderPropertyValues()
	{
		return Utils.createArrayWithLeadingNull(VoicemailListEntryProperties.values());
	}
	
	public OrderDirection getOrderDirection()
	{
		return orderDirection;
	}
	
	public static final String PROPERTY_ORDER_DIRECTION = "orderDirection";
	
	public void setOrderDirection(OrderDirection orderDirection)
	{
		OrderDirection old = this.orderDirection;
		this.orderDirection = orderDirection;
		changeSupport.firePropertyChange(PROPERTY_ORDER_DIRECTION, old, orderDirection);
	}
	
	public OrderDirection[] getOrderDirectionValues()
	{
		return Utils.createArrayWithLeadingNull(OrderDirection.values());
	}
	
	public int getCountOffset()
	{
		return countOffset;
	}
	
	public static final String PROPERTY_COUNT_OFFSET = "countOffset";
	
	public void setCountOffset(int countOffset)
	{
		int old = this.countOffset;
		this.countOffset = countOffset;
		changeSupport.firePropertyChange(PROPERTY_COUNT_OFFSET, old, countOffset);
	}
	
	public int getCountLimit()
	{
		return countLimit;
	}
	
	public static final String PROPERTY_COUNT_LIMIT = "countLimit";
	
	public void setCountLimit(int countLimit)
	{
		int old = this.countLimit;
		this.countLimit = countLimit;
		changeSupport.firePropertyChange(PROPERTY_COUNT_LIMIT, old, countLimit);
	}
	
	public List<Integer> getSelectedRows()
	{
		return selectedRows;
	}
	
	public VoicemailListEntryFolder getDestinationFolder()
	{
		return destinationFolder;
	}
	
	public static final String PROPERTY_DESTINATION_FOLDER = "destinationFolder";
	
	public void setDestinationFolder(VoicemailListEntryFolder destinationFolder)
	{
		VoicemailListEntryFolder old = this.destinationFolder;
		this.destinationFolder = destinationFolder;
		changeSupport.firePropertyChange(PROPERTY_DESTINATION_FOLDER, old, destinationFolder);
	}
	
	public VoicemailListEntryFolder[] getDestinationFolderValues()
	{
		return VoicemailListEntryFolder.values();
	}
	
	public List<Mailbox> getMailboxRestrictionValues()
	{
		return mailboxRestrictionValues;
	}
	
	public static final String PROPERTY_MAILBOX_RESTRICTION_VALUES = "mailboxRestrictionValues";
	
	public void updateMailboxRestrictionValues()
	{
		List<Mailbox> old = mailboxRestrictionValues;
		mailboxRestrictionValues = new LinkedList<Mailbox>();
		mailboxRestrictionValues.add(null);
		if (mailboxValues != null) mailboxRestrictionValues.addAll(mailboxValues);
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_MAILBOX_RESTRICTION_VALUES, old,
			mailboxRestrictionValues);
	}
	
	public Mailbox getMailboxRestriction()
	{
		return mailboxRestriction;
	}
	
	public static final String PROPERTY_MAILBOX_RESTRICTION = "mailboxRestriction";
	
	public void setMailboxRestriction(Mailbox mailboxRestriction)
	{
		Mailbox old = this.mailboxRestriction;
		this.mailboxRestriction = mailboxRestriction;
		changeSupport.firePropertyChange(PROPERTY_MAILBOX_RESTRICTION, old, mailboxRestriction);
	}
	
	public List<Mailbox> getMailboxValues()
	{
		return mailboxValues;
	}
	
	public static final String PROPERTY_MAILBOX_VALUES = "mailboxValues";
	
	public void setMailboxValues(List<Mailbox> mailboxValues)
	{
		List<Mailbox> old = this.mailboxValues;
		this.mailboxValues = mailboxValues;
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_MAILBOX_VALUES, old, mailboxValues);
		updateMailboxRestrictionValues();
		updateSelectedMailbox();
		if (this.callViewModel != null)
		{
			this.callViewModel.setMailboxValues(mailboxValues);
		}
	}
	
	private void updateSelectedMailbox()
	{
		if (mailboxValues != null && !mailboxValues.contains(selectedMailbox))
		{
			if (mailboxValues.isEmpty())
			{
				setSelectedMailbox(null);
			}
			else
			{
				setSelectedMailbox(mailboxValues.get(0));
			}
		}
	}
	
	public Mailbox getSelectedMailbox()
	{
		return selectedMailbox;
	}
	
	public static final String PROPERTY_SELECTED_MAILBOX = "selectedMailbox";
	
	public void setSelectedMailbox(Mailbox selectedMailbox)
	{
		Mailbox old = this.selectedMailbox;
		this.selectedMailbox = selectedMailbox;
		if (changeSupport != null) changeSupport.firePropertyChange(PROPERTY_SELECTED_MAILBOX, old,
			selectedMailbox);
	}
	
	public ActionListener getGetGetMailboxesActionListener()
	{
		return getGetMailboxesActionListener;
	}
	
	public String getOriginatingNumber()
	{
		return originatingNumber;
	}
	
	public static final String PROPERTY_ORIGINATING_NUMBER = "originatingNumber";
	
	public void setOriginatingNumber(String originatingNumber)
	{
		String old = this.originatingNumber;
		this.originatingNumber = originatingNumber;
		changeSupport.firePropertyChange(PROPERTY_ORIGINATING_NUMBER, old, originatingNumber);
	}
	
	public ActionListener getCallMailboxWithPhoneActionListener()
	{
		return callMailboxWithPhoneActionListener;
	}
	
	public ActionListener getCallMailboxWithNumberActionListener()
	{
		return callMailboxWithNumberActionListener;
	}
	
	public ActionListener getCallVoicemailWithNumberActionListener()
	{
		return callVoicemailWithNumberActionListener;
	}
	
	public ActionListener getCallVoicemailWithPhoneActionListener()
	{
		return callVoicemailWithPhoneActionListener;
	}
	
	public ActionListener getGetVoicemailListActionListener()
	{
		return getVoicemailListActionListener;
	}
	
	public ActionListener getDeleteVoicemailListEntryActionListener()
	{
		return deleteVoicemailListEntryActionListener;
	}
	
	public ActionListener getGetVoicemailFileUrlActionListener()
	{
		return getVoicemailFileUrlActionListener;
	}
	
	public ActionListener getMoveVoicemailListEntryActionListener()
	{
		return moveVoicemailListEntryActionListener;
	}
	
	public ActionListener getDeleteVoicemailListEntriesActionListener()
	{
		return deleteVoicemailListEntriesActionListener;
	}
	
	public ActionListener getMoveVoicemailListEntriesActionListener()
	{
		return moveVoicemailListEntriesActionListener;
	}
	
	public ActionListener getGetVoicemailFileActionListener()
	{
		return getVoicemailFileActionListener;
	}
	
	public ActionListener getBrowseVoicemailFileUrlActionListener()
	{
		return browseVoicemailFileUrlActionListener;
	}
	
	public ActionListener getOpenVoicemailFileActionListener()
	{
		return openVoicemailFileActionListener;
	}
}
