/*
 * $Id: CallListEntryProperties.java 466 2011-01-18 18:25:29Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/CallListEntryProperties.java $
 * 
 * $LastChangedDate: 2011-01-18 19:25:29 +0100 (Di, 18 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 466 $
 */
package de.starface.integration.uci.java.v22.types;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v22.messages.requests.UciFaxListRequests;
import de.starface.integration.uci.java.v22.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v22.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v22.values.GroupRestriction;
import de.starface.integration.uci.java.v22.values.OrderDirection;

/**
 * This object holds the properties of the fax list as returned by
 * {@link UciFaxListRequests#getFaxList(Date, Date, FaxListEntryFolder, String, FaxListEntryProperties, OrderDirection, int, int)}
 * 
 * @author tilmann
 */
@RpcValueObject
public class FaxList implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	/** The entries received before this point of time given */
	@RpcValue
	private Date receivedBefore;
	
	/** The entries received after this point of time given */
	@RpcValue
	private Date receivedAfter;
	
	/**
	 * The entries are restricted to those in the given folder with one of the values of
	 * {@link FaxListEntryFolder}
	 */
	@RpcValue
	private FaxListEntryFolder folderRestriction;
	
	/**
	 * The entries are restricted to faxes belonging to the group with the given id or to faxes that
	 * do not belong to any group if {@link GroupRestriction#NON_GROUP} is given. If an empty String
	 * is given no restriction is used.
	 */
	@RpcValue
	private String groupRestriction = "";
	
	/**
	 * The entries are ordered by the given property. This may be one of the properties in
	 * {@link FaxListEntryProperties}
	 */
	@RpcValue
	private FaxListEntryProperties orderProperty;
	
	/**
	 * The entries are ordered in the given direction. This may be one of the values in
	 * {@link OrderDirection}
	 */
	@RpcValue
	private OrderDirection orderDirection;
	
	/**
	 * The offset used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countOffset;
	
	/**
	 * The limit used for the retrieval of this list regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int countLimit;
	
	/**
	 * The total count of available entries on the server regarding the given time constraints and
	 * restrictions
	 */
	@RpcValue
	private int totalCount;
	
	/**
	 * The fax list entries in this list
	 */
	@RpcValue
	private List<FaxListEntry> entries;
	
	public Date getReceivedBefore()
	{
		return receivedBefore;
	}
	
	public void setReceivedBefore(Date receivedBefore)
	{
		this.receivedBefore = receivedBefore;
	}
	
	public Date getReceivedAfter()
	{
		return receivedAfter;
	}
	
	public void setReceivedAfter(Date receivedAfter)
	{
		this.receivedAfter = receivedAfter;
	}
	
	public FaxListEntryFolder getFolderRestriction()
	{
		return folderRestriction;
	}
	
	public void setFolderRestriction(FaxListEntryFolder folderRestriction)
	{
		this.folderRestriction = folderRestriction;
	}
	
	public FaxListEntryProperties getOrderProperty()
	{
		return orderProperty;
	}
	
	public void setOrderProperty(FaxListEntryProperties orderProperty)
	{
		this.orderProperty = orderProperty;
	}
	
	public OrderDirection getOrderDirection()
	{
		return orderDirection;
	}
	
	public void setOrderDirection(OrderDirection orderDirection)
	{
		this.orderDirection = orderDirection;
	}
	
	public int getCountOffset()
	{
		return countOffset;
	}
	
	public void setCountOffset(int countOffset)
	{
		this.countOffset = countOffset;
	}
	
	public int getCountLimit()
	{
		return countLimit;
	}
	
	public void setCountLimit(int countLimit)
	{
		this.countLimit = countLimit;
	}
	
	public int getTotalCount()
	{
		return totalCount;
	}
	
	public void setTotalCount(int totalCount)
	{
		this.totalCount = totalCount;
	}
	
	public List<FaxListEntry> getEntries()
	{
		return entries;
	}
	
	public void setEntries(List<FaxListEntry> entries)
	{
		this.entries = entries;
	}
	
	public String getGroupRestriction()
	{
		return groupRestriction;
	}
	
	public void setGroupRestriction(String groupRestriction)
	{
		this.groupRestriction = groupRestriction;
	}
	
	@Override
	public String toString()
	{
		return "FaxList [receivedBefore=" + receivedBefore + ", receivedAfter=" + receivedAfter
			+ ", folderRestriction=" + folderRestriction + ", orderProperty=" + orderProperty
			+ ", orderDirection=" + orderDirection + ", countOffset=" + countOffset
			+ ", countLimit=" + countLimit + ", totalCount=" + totalCount + ", entries=" + entries
			+ "]";
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + countLimit;
		result = prime * result + countOffset;
		result = prime * result + ((entries == null) ? 0 : entries.hashCode());
		result = prime * result + ((folderRestriction == null) ? 0 : folderRestriction.hashCode());
		result = prime * result + ((orderDirection == null) ? 0 : orderDirection.hashCode());
		result = prime * result + ((orderProperty == null) ? 0 : orderProperty.hashCode());
		result = prime * result + ((receivedAfter == null) ? 0 : receivedAfter.hashCode());
		result = prime * result + ((receivedBefore == null) ? 0 : receivedBefore.hashCode());
		result = prime * result + totalCount;
		return result;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		FaxList other = (FaxList) obj;
		if (countLimit != other.countLimit) return false;
		if (countOffset != other.countOffset) return false;
		if (entries == null)
		{
			if (other.entries != null) return false;
		}
		else if (!entries.equals(other.entries)) return false;
		if (folderRestriction != other.folderRestriction) return false;
		if (orderDirection != other.orderDirection) return false;
		if (orderProperty != other.orderProperty) return false;
		if (receivedAfter == null)
		{
			if (other.receivedAfter != null) return false;
		}
		else if (!receivedAfter.equals(other.receivedAfter)) return false;
		if (receivedBefore == null)
		{
			if (other.receivedBefore != null) return false;
		}
		else if (!receivedBefore.equals(other.receivedBefore)) return false;
		if (totalCount != other.totalCount) return false;
		return true;
	}
	
}
