/*
 * $Id: FaxListEntryProperties.java 482 2011-01-25 15:21:54Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/FaxListEntryProperties.java $
 * 
 * $LastChangedDate: 2011-01-25 16:21:54 +0100 (Di, 25 Jan 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 482 $
 */
package de.starface.integration.uci.java.v22.types;

import java.io.Serializable;
import java.util.Date;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.com.rpc.annotation.RpcValues;
import de.starface.integration.uci.java.v22.messages.events.UciFaxEvents;
import de.starface.integration.uci.java.v22.messages.requests.UciFaxListRequests;
import de.starface.integration.uci.java.v22.messages.requests.UciFaxRequests;
import de.starface.integration.uci.java.v22.values.FaxListEntryFolder;
import de.starface.integration.uci.java.v22.values.FaxListEntryProperties;
import de.starface.integration.uci.java.v22.values.OrderDirection;

/**
 * This object holds the properties of the fax list entries as returned by
 * {@link UciFaxListRequests#getFaxList(Date, Date, FaxListEntryFolder, String, FaxListEntryProperties, OrderDirection, int, int)}
 * 
 * @author tilmann
 */
@RpcValueObject
public class FaxListEntry implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * The id of the fax list entry
	 * 
	 * ATTENTION: This is not the same id as used in the fax functions of {@link UciFaxRequests} and
	 * {@link UciFaxEvents}
	 */
	@RpcValue
	private String id;
	
	/** The number that was called, if known */
	@RpcValue
	private String calledNumber = "";
	
	/** The number of the sender, if known */
	@RpcValue
	private String callerNumber = "";
	
	/** A description of the fax call (usually the CLIP), if known */
	@RpcValue
	private String callDescription = "";
	
	/**
	 * The folder the fax belongs to holding one of the values defined in {@link FaxListEntryFolder}
	 * .
	 */
	@RpcValue
	private FaxListEntryFolder folder;
	
	/**
	 * The time when the fax was received. Dates will be ordered newest first per default.
	 */
	@RpcValue
	private Date receptionTime;
	
	/** A flag indicating that the fax was not transmitted completely */
	
	@RpcValues({ @RpcValue(value = "isPartial", minVersion = 22), @RpcValue(maxVersion = 21) })
	private boolean partial;
	
	/**
	 * The name of the group if this fax was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	private String groupId = "";
	
	@Override
	public String toString()
	{
		return "FaxListEntry [id=" + id + ", calledNumber=" + calledNumber + ", callerNumber="
			+ callerNumber + ", callDescription=" + callDescription + ", folder=" + folder
			+ ", receptionTime=" + receptionTime + ", partial=" + partial + ", groupId=" + groupId
			+ "]";
	}
	
	public String getId()
	{
		return id;
	}
	
	public void setId(String id)
	{
		this.id = id;
	}
	
	public String getCalledNumber()
	{
		return calledNumber;
	}
	
	public void setCalledNumber(String calledNumber)
	{
		this.calledNumber = calledNumber;
	}
	
	public String getCallerNumber()
	{
		return callerNumber;
	}
	
	public void setCallerNumber(String callerNumber)
	{
		this.callerNumber = callerNumber;
	}
	
	public String getCallDescription()
	{
		return callDescription;
	}
	
	public void setCallDescription(String callDescription)
	{
		this.callDescription = callDescription;
	}
	
	public FaxListEntryFolder getFolder()
	{
		return folder;
	}
	
	public void setFolder(FaxListEntryFolder folder)
	{
		this.folder = folder;
	}
	
	public Date getReceptionTime()
	{
		return receptionTime;
	}
	
	public void setReceptionTime(Date receptionTime)
	{
		this.receptionTime = receptionTime;
	}
	
	public boolean isPartial()
	{
		return partial;
	}
	
	public void setPartial(boolean partial)
	{
		this.partial = partial;
	}
	
	public String getGroupId()
	{
		return groupId;
	}
	
	public void setGroupId(String groupId)
	{
		this.groupId = groupId;
	}
	
	public boolean isGroupFax()
	{
		return groupId != null && !"".equals(groupId);
	}
	
	public boolean isIncomingFax()
	{
		return folder != FaxListEntryFolder.SENT;
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((callDescription == null) ? 0 : callDescription.hashCode());
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((callerNumber == null) ? 0 : callerNumber.hashCode());
		result = prime * result + ((folder == null) ? 0 : folder.hashCode());
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + (partial ? 1231 : 1237);
		result = prime * result + ((receptionTime == null) ? 0 : receptionTime.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		FaxListEntry other = (FaxListEntry) obj;
		if (callDescription == null)
		{
			if (other.callDescription != null) return false;
		}
		else if (!callDescription.equals(other.callDescription)) return false;
		if (calledNumber == null)
		{
			if (other.calledNumber != null) return false;
		}
		else if (!calledNumber.equals(other.calledNumber)) return false;
		if (callerNumber == null)
		{
			if (other.callerNumber != null) return false;
		}
		else if (!callerNumber.equals(other.callerNumber)) return false;
		if (folder != other.folder) return false;
		if (groupId == null)
		{
			if (other.groupId != null) return false;
		}
		else if (!groupId.equals(other.groupId)) return false;
		if (id == null)
		{
			if (other.id != null) return false;
		}
		else if (!id.equals(other.id)) return false;
		if (partial != other.partial) return false;
		if (receptionTime == null)
		{
			if (other.receptionTime != null) return false;
		}
		else if (!receptionTime.equals(other.receptionTime)) return false;
		return true;
	}
	
}
