package de.starface.integration.uci.java.v22.types;

import java.io.Serializable;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v22.messages.requests.UciFunctionKeyRequests;
import de.starface.integration.uci.java.v22.values.FunctionKeyState;
import de.starface.integration.uci.java.v22.values.FunctionKeyType;

/**
 * This object holds the properties of the function keys as returned by
 * {@link UciFunctionKeyRequests#getFunctionKeys()}.
 * 
 * @author tilmann
 */
@RpcValueObject
public class FunctionKey implements Positionable, Serializable
{
	private static final long serialVersionUID = 1L;
	
	/** The id of the function key */
	@RpcValue
	private String id;
	
	/** The display name of the function key */
	@RpcValue
	private String name;
	
	/** The state of the function key with one of the constants of {@link FunctionKeyState} */
	@RpcValue
	private FunctionKeyState state;
	
	/** The type of the function key with one of the constants of {@link FunctionKeyType} */
	@RpcValue
	private FunctionKeyType type;
	
	/**
	 * The user state of the function key if the key is of type
	 * {@link FunctionKeyType#BUSY_LAMP_FIELD}
	 */
	@RpcValue
	private UserState userState;
	
	/**
	 * The hash of the key's image if an image is assigned to the key the image itself can then be
	 * requested using {@link UciFunctionKeyRequests#getImageForKey(String)}
	 */
	@RpcValue
	private String imageHash = "";
	
	/** The position of the key in the users function key settings */
	@RpcValue
	private int position;
	
	@Override
	public String toString()
	{
		return "FunctionKey [id=" + id + ", name=" + name + ", state=" + state + ", type=" + type + ", userState="
			+ userState + ", imageHash=" + imageHash + ", position=" + position + "]";
	}
	
	public String getId()
	{
		return id;
	}
	
	public void setId(String id)
	{
		this.id = id;
	}
	
	public String getName()
	{
		return name;
	}
	
	public void setName(String name)
	{
		this.name = name;
	}
	
	public FunctionKeyState getState()
	{
		return state;
	}
	
	public void setState(FunctionKeyState state)
	{
		this.state = state;
	}
	
	public FunctionKeyType getType()
	{
		return type;
	}
	
	public void setType(FunctionKeyType type)
	{
		this.type = type;
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((imageHash == null) ? 0 : imageHash.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + position;
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		result = prime * result + ((userState == null) ? 0 : userState.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		FunctionKey other = (FunctionKey) obj;
		if (id == null)
		{
			if (other.id != null) return false;
		}
		else if (!id.equals(other.id)) return false;
		if (imageHash == null)
		{
			if (other.imageHash != null) return false;
		}
		else if (!imageHash.equals(other.imageHash)) return false;
		if (name == null)
		{
			if (other.name != null) return false;
		}
		else if (!name.equals(other.name)) return false;
		if (position != other.position) return false;
		if (state != other.state) return false;
		if (type != other.type) return false;
		if (userState == null)
		{
			if (other.userState != null) return false;
		}
		else if (!userState.equals(other.userState)) return false;
		return true;
	}
	
	public UserState getUserState()
	{
		return userState;
	}
	
	public void setUserState(UserState userState)
	{
		this.userState = userState;
	}
	
	public String getImageHash()
	{
		return imageHash;
	}
	
	public void setImageHash(String imageHash)
	{
		this.imageHash = imageHash;
	}
	
	public int getPosition()
	{
		return position;
	}
	
	public void setPosition(int position)
	{
		this.position = position;
	}
	
}