package de.starface.integration.uci.java.v22.types;

import java.io.Serializable;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v22.messages.requests.UciPhoneRequests;

/**
 * This object holds the properties of the phone numbers as returned by
 * {@link UciPhoneRequests#getAvailableDisplayNumbers()}
 * 
 * @author tilmann
 */
@RpcValueObject
public class PhoneNumber implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	/** The id of the phone number */
	@RpcValue
	private String id;
	
	/** The number of the phone number */
	@RpcValue
	private String number;
	
	/**
	 * The id of the group if this phone number belongs to a group (the empty string otherwise)
	 */
	@RpcValue
	private String groupId = "";
	
	@Override
	public String toString()
	{
		return "PhoneNumber [id=" + id + ", number=" + number + ", groupId=" + groupId + "]";
	}
	
	public String getId()
	{
		return id;
	}
	
	public void setId(String id)
	{
		this.id = id;
	}
	
	public String getGroupId()
	{
		return groupId;
	}
	
	public void setGroupId(String groupId)
	{
		this.groupId = groupId;
	}
	
	public boolean isGroupNumber()
	{
		return groupId != null && !"".equals(groupId);
	}
	
	public String getNumber()
	{
		return number;
	}
	
	public void setNumber(String number)
	{
		this.number = number;
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((number == null) ? 0 : number.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		PhoneNumber other = (PhoneNumber) obj;
		if (groupId == null)
		{
			if (other.groupId != null) return false;
		}
		else if (!groupId.equals(other.groupId)) return false;
		if (id == null)
		{
			if (other.id != null) return false;
		}
		else if (!id.equals(other.id)) return false;
		if (number == null)
		{
			if (other.number != null) return false;
		}
		else if (!number.equals(other.number)) return false;
		return true;
	}
	
}
