/*
 * $Id: UcpCallEvents.java 498 2011-02-18 18:13:49Z tilmann $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/client/UcpClientCommunicationCall.java $
 * 
 * $LastChangedDate: 2011-02-18 19:13:49 +0100 (Fr, 18. Feb 2011) $ $LastChangedBy: tilmann $
 * $LastChangedRevision: 498 $
 */
package de.starface.integration.uci.java.v22.ucp.messages.events;

import java.util.Map;

import de.starface.com.rpc.annotation.RpcAsynchronous;
import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v22.UciService;
import de.starface.integration.uci.java.v22.ucp.messages.requests.UcpCallRequests;
import de.starface.integration.uci.java.v22.values.CallProperties;
import de.starface.integration.uci.java.v22.values.CallState;

/**
 * This interface defines the call events that can be received by a UCI client implementation in
 * order to handle phone calls.
 * 
 * @author tilmann
 */
@RpcInterface(UcpCallEvents.SERVICE_NAME)
@UciService(UcpCallEvents.SERVICE_NAME)
public interface UcpCallEvents
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v22.events.call";
	
	/**
	 * The ReceiveCallState event provides a number of properties about the call the state is
	 * changing of. The information provided in the map is defined by its properties as seen in
	 * {@link CallProperties}. If a name or number is not known to STARFACE an empty string is
	 * committed for that property. It is possible that information about the caller or the called
	 * person gets available in the middle of a series of events so that the first call state events
	 * don’t provide any details but later events contain full participant information. The
	 * sequential change of the {@link CallProperties#state} property adheres to the state diagram
	 * given in the UCI specification. The call state {@link CallState#REQUESTED} is never signaled
	 * by the server. It is assumed after a successful return of the
	 * {@link UcpCallRequests#placeCallWithPhone(String, String, String)} request.
	 * 
	 * @param callProperties
	 *        the different properties of the call in a {@link Map}. The constants of
	 *        {@link CallProperties} are used as keys in the map.
	 */
	@RpcAsynchronous
	public void receiveCallState(Map<String, Object> callProperties);
}
