package de.starface.integration.uci.java.v30.types;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.events.UciCallEvents;
import de.starface.integration.uci.java.v30.values.CallState;
import de.starface.integration.uci.java.v30.values.ForwardType;

/**
 * This object holds the properties of the call as returned by
 * {@link UciCallEvents#receiveCallState(Call)}.
 * 
 * @author tilmann
 */
@RpcValueObject
public class Call implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	/** The id of the call in form of a UUID as {@link String} */
	@RpcValue
	protected String id;
	
	/** The state of the call */
	@RpcValue
	protected CallState state;
	
	/** The point of time when the call reached the state */
	@RpcValue
	protected Date timestamp;
	
	/** The phone number that has been called */
	@RpcValue
	protected String calledNumber = "";
	
	/** The name of the person being called */
	@RpcValue
	protected String calledName = "";
	
	/** The number of the caller */
	@RpcValue
	protected String callerNumber = "";
	
	/** The name of the person calling */
	@RpcValue
	protected String callerName = "";
	
	/**
	 * The id of the group if this call was received by a group (the empty string otherwise)
	 */
	@RpcValue(minVersion = 22)
	protected String groupId = "";
	
	/** Name of the device **/
	@RpcValue(minVersion = 30)
	protected List<String> peerNames;
	
	/** The channel name **/
	@RpcValue(minVersion = 30)
	protected List<String> channelNames;
	
	/** The sip callId **/
	@RpcValue(minVersion = 30)
	protected List<String> sipCallIds;
	
	/** Is caller **/
	@RpcValue(minVersion = 30)
	protected Boolean caller = Boolean.FALSE;
	
	/** The point of time when the call is connected */
	@RpcValue(minVersion = 30)
	protected Date connectedTimestamp;
	
	/** Is in the current call recording activated */
	@RpcValue(minVersion = 30)
	protected Boolean recorded = Boolean.FALSE;
	
	/** Is in the current call recorded by the user */
	@RpcValue(minVersion = 30)
	protected Boolean recorder = Boolean.FALSE;
	
	/** Is the user a monitor owner */
	@RpcValue(minVersion = 30)
	protected Boolean recordOwner = Boolean.FALSE;
	
	/** AvatarHash of Caller */
	@RpcValue(minVersion = 30)
	protected String avatarHash = "";
	
	/** Doorline Cam of Caller */
	@RpcValue(minVersion = 30)
	protected String doorlineCamUrl = "";
	
	/** Doorline dtm code */
	@RpcValue(minVersion = 30)
	protected String doorlineDtmfCode = "";
	
	/** Doorline image provider id */
	@RpcValue(minVersion = 30)
	protected Integer doorlineImageProviderId;
	
	/** Forward type */
	@RpcValue(minVersion = 30)
	protected ForwardType forwardType = ForwardType.NONE;
	
	/** Forwarder callerIdName of farwarder */
	@RpcValue(minVersion = 30)
	protected String forwarderCallerIdName = "";
	
	/** Forwarder callerIdNumber of farwarder */
	@RpcValue(minVersion = 30)
	protected String forwarderCallerIdNumber = "";
	
	/** The conference room identifikation */
	@RpcValue(minVersion = 30)
	protected String conferenceRoomId = "";
	
	/** The current call is an conference admin */
	@RpcValue(minVersion = 30)
	protected boolean conferenceAdmin = false;
	
	/** The reference of the outher side of consultation */
	@RpcValue(minVersion = 30)
	protected String referenceOfConsultation = "";
	
	/** The time between now and the connected timestamp/timestamp */
	@RpcValue(minVersion = 30)
	protected long duration = 0;
	
	/** Is this call a consultationCall */
	@RpcValue(minVersion = 30)
	protected boolean consultationCall = false;
	
	/** The jabber id */
	@RpcValue(minVersion = 30)
	protected String jabberId = "";
	
	public Date getConnectedTimestamp()
	{
		return connectedTimestamp;
	}
	
	public void setConnectedTimestamp(Date connectedTimestamp)
	{
		this.connectedTimestamp = connectedTimestamp;
	}
	
	public String getId()
	{
		return id;
	}
	
	public void setId(String id)
	{
		this.id = id;
	}
	
	public CallState getState()
	{
		return state;
	}
	
	public void setState(CallState state)
	{
		this.state = state;
	}
	
	public Date getTimestamp()
	{
		return timestamp;
	}
	
	public void setTimestamp(Date timestamp)
	{
		this.timestamp = timestamp;
	}
	
	public String getCalledNumber()
	{
		return calledNumber;
	}
	
	public void setCalledNumber(String calledNumber)
	{
		this.calledNumber = calledNumber;
	}
	
	public String getCalledName()
	{
		return calledName;
	}
	
	public void setCalledName(String calledName)
	{
		this.calledName = calledName;
	}
	
	public String getCallerNumber()
	{
		return callerNumber;
	}
	
	public void setCallerNumber(String callerNumber)
	{
		this.callerNumber = callerNumber;
	}
	
	public String getCallerName()
	{
		return callerName;
	}
	
	public void setCallerName(String callerName)
	{
		this.callerName = callerName;
	}
	
	public List<String> getChannelNames()
	{
		return channelNames;
	}
	
	public void setChannelNames(List<String> channelNames)
	{
		this.channelNames = channelNames;
	}
	
	public String getGroupId()
	{
		return groupId;
	}
	
	public void setGroupId(String groupId)
	{
		this.groupId = groupId;
	}
	
	public List<String> getPeerNames()
	{
		return peerNames;
	}
	
	public void setPeerNames(List<String> peerNames)
	{
		this.peerNames = peerNames;
	}
	
	public boolean isGroupCall()
	{
		return groupId != null && !"".equals(groupId);
	}
	
	public void setRecorder(Boolean recorder)
	{
		this.recorder = recorder;
	}
	
	public void setRecorded(Boolean recorded)
	{
		this.recorded = recorded;
	}
	
	public void setRecordOwner(Boolean recordOwner)
	{
		this.recordOwner = recordOwner;
	}
	
	public String getAvatarHash()
	{
		return avatarHash;
	}
	
	public void setAvatarHash(String avatarHash)
	{
		this.avatarHash = avatarHash;
	}
	
	public String getDoorlineCamUrl()
	{
		return doorlineCamUrl;
	}
	
	public void setDoorlineCamUrl(String doorlineCamUrl)
	{
		this.doorlineCamUrl = doorlineCamUrl;
	}
	
	public String getDoorlineDtmfCode()
	{
		return doorlineDtmfCode;
	}
	
	public void setDoorlineDtmfCode(String doorlineDtmfCode)
	{
		this.doorlineDtmfCode = doorlineDtmfCode;
	}
	
	public Integer getDoorlineImageProviderId()
	{
		return doorlineImageProviderId;
	}
	
	public void setDoorlineImageProviderId(Integer doorlineImageProviderId)
	{
		this.doorlineImageProviderId = doorlineImageProviderId;
	}
	
	public List<String> getSipCallIds()
	{
		return sipCallIds;
	}
	
	public void setSipCallIds(List<String> sipCallIds)
	{
		this.sipCallIds = sipCallIds;
	}
	
	public Boolean isCaller()
	{
		return caller;
	}
	
	public void setCaller(Boolean caller)
	{
		this.caller = caller;
	}
	
	public ForwardType getForwardType()
	{
		return forwardType;
	}
	
	public void setForwardType(ForwardType forwardType)
	{
		this.forwardType = forwardType;
	}
	
	public String getForwarderCallerIdName()
	{
		return forwarderCallerIdName;
	}
	
	public void setForwarderCallerIdName(String forwarderCallerIdName)
	{
		this.forwarderCallerIdName = forwarderCallerIdName;
	}
	
	public String getForwarderCallerIdNumber()
	{
		return forwarderCallerIdNumber;
	}
	
	public void setForwarderCallerIdNumber(String forwarderCallerIdNumber)
	{
		this.forwarderCallerIdNumber = forwarderCallerIdNumber;
	}
	
	public String getConferenceRoomId()
	{
		return conferenceRoomId;
	}
	
	public void setConferenceRoomId(String conferenceRoomId)
	{
		this.conferenceRoomId = conferenceRoomId;
	}
	
	public Boolean isRecorder()
	{
		return recorder;
	}
	
	public Boolean isRecorded()
	{
		return recorded;
	}
	
	public Boolean isRecordOwner()
	{
		return recordOwner;
	}
	
	public boolean isConferenceAdmin()
	{
		return conferenceAdmin;
	}
	
	public void setConferenceAdmin(boolean conferenceAdmin)
	{
		this.conferenceAdmin = conferenceAdmin;
	}
	
	public long getDuration()
	{
		return duration;
	}
	
	public void setDuration(long duration)
	{
		this.duration = duration;
	}
	
	public boolean isConsultationCall()
	{
		return consultationCall;
	}
	
	public void setConsultationCall(boolean consultationCall)
	{
		this.consultationCall = consultationCall;
	}
	
	public String getReferenceOfConsultation()
	{
		return referenceOfConsultation;
	}
	
	public void setReferenceOfConsultation(String referenceOfConsultation)
	{
		this.referenceOfConsultation = referenceOfConsultation;
	}
	
	/**
	 * @return the jabberId
	 */
	public String getJabberId()
	{
		return jabberId;
	}
	
	/**
	 * @param jabberId
	 *        the jabberId to set
	 */
	public void setJabberId(String jabberId)
	{
		this.jabberId = jabberId;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + ((avatarHash == null) ? 0 : avatarHash.hashCode());
		result = prime * result + ((calledName == null) ? 0 : calledName.hashCode());
		result = prime * result + ((calledNumber == null) ? 0 : calledNumber.hashCode());
		result = prime * result + ((caller == null) ? 0 : caller.hashCode());
		result = prime * result + ((callerName == null) ? 0 : callerName.hashCode());
		result = prime * result + ((callerNumber == null) ? 0 : callerNumber.hashCode());
		result = prime * result + ((channelNames == null) ? 0 : channelNames.hashCode());
		result = prime * result + (conferenceAdmin ? 1231 : 1237);
		result = prime * result + ((conferenceRoomId == null) ? 0 : conferenceRoomId.hashCode());
		result = prime * result + ((connectedTimestamp == null) ? 0 : connectedTimestamp.hashCode());
		result = prime * result + (consultationCall ? 1231 : 1237);
		result = prime * result + ((doorlineCamUrl == null) ? 0 : doorlineCamUrl.hashCode());
		result = prime * result + ((doorlineDtmfCode == null) ? 0 : doorlineDtmfCode.hashCode());
		result = prime * result + ((doorlineImageProviderId == null) ? 0 : doorlineImageProviderId.hashCode());
		result = prime * result + (int) (duration ^ (duration >>> 32));
		result = prime * result + ((forwardType == null) ? 0 : forwardType.hashCode());
		result = prime * result + ((forwarderCallerIdName == null) ? 0 : forwarderCallerIdName.hashCode());
		result = prime * result + ((forwarderCallerIdNumber == null) ? 0 : forwarderCallerIdNumber.hashCode());
		result = prime * result + ((groupId == null) ? 0 : groupId.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((jabberId == null) ? 0 : jabberId.hashCode());
		result = prime * result + ((peerNames == null) ? 0 : peerNames.hashCode());
		result = prime * result + ((recordOwner == null) ? 0 : recordOwner.hashCode());
		result = prime * result + ((recorded == null) ? 0 : recorded.hashCode());
		result = prime * result + ((recorder == null) ? 0 : recorder.hashCode());
		result = prime * result + ((referenceOfConsultation == null) ? 0 : referenceOfConsultation.hashCode());
		result = prime * result + ((sipCallIds == null) ? 0 : sipCallIds.hashCode());
		result = prime * result + ((state == null) ? 0 : state.hashCode());
		result = prime * result + ((timestamp == null) ? 0 : timestamp.hashCode());
		return result;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Call other = (Call) obj;
		if (avatarHash == null)
		{
			if (other.avatarHash != null) return false;
		}
		else if (!avatarHash.equals(other.avatarHash)) return false;
		if (calledName == null)
		{
			if (other.calledName != null) return false;
		}
		else if (!calledName.equals(other.calledName)) return false;
		if (calledNumber == null)
		{
			if (other.calledNumber != null) return false;
		}
		else if (!calledNumber.equals(other.calledNumber)) return false;
		if (caller == null)
		{
			if (other.caller != null) return false;
		}
		else if (!caller.equals(other.caller)) return false;
		if (callerName == null)
		{
			if (other.callerName != null) return false;
		}
		else if (!callerName.equals(other.callerName)) return false;
		if (callerNumber == null)
		{
			if (other.callerNumber != null) return false;
		}
		else if (!callerNumber.equals(other.callerNumber)) return false;
		if (channelNames == null)
		{
			if (other.channelNames != null) return false;
		}
		else if (!channelNames.equals(other.channelNames)) return false;
		if (conferenceAdmin != other.conferenceAdmin) return false;
		if (conferenceRoomId == null)
		{
			if (other.conferenceRoomId != null) return false;
		}
		else if (!conferenceRoomId.equals(other.conferenceRoomId)) return false;
		if (connectedTimestamp == null)
		{
			if (other.connectedTimestamp != null) return false;
		}
		else if (!connectedTimestamp.equals(other.connectedTimestamp)) return false;
		if (consultationCall != other.consultationCall) return false;
		if (doorlineCamUrl == null)
		{
			if (other.doorlineCamUrl != null) return false;
		}
		else if (!doorlineCamUrl.equals(other.doorlineCamUrl)) return false;
		if (doorlineDtmfCode == null)
		{
			if (other.doorlineDtmfCode != null) return false;
		}
		else if (!doorlineDtmfCode.equals(other.doorlineDtmfCode)) return false;
		if (doorlineImageProviderId == null)
		{
			if (other.doorlineImageProviderId != null) return false;
		}
		else if (!doorlineImageProviderId.equals(other.doorlineImageProviderId)) return false;
		if (duration != other.duration) return false;
		if (forwardType != other.forwardType) return false;
		if (forwarderCallerIdName == null)
		{
			if (other.forwarderCallerIdName != null) return false;
		}
		else if (!forwarderCallerIdName.equals(other.forwarderCallerIdName)) return false;
		if (forwarderCallerIdNumber == null)
		{
			if (other.forwarderCallerIdNumber != null) return false;
		}
		else if (!forwarderCallerIdNumber.equals(other.forwarderCallerIdNumber)) return false;
		if (groupId == null)
		{
			if (other.groupId != null) return false;
		}
		else if (!groupId.equals(other.groupId)) return false;
		if (id == null)
		{
			if (other.id != null) return false;
		}
		else if (!id.equals(other.id)) return false;
		if (jabberId == null)
		{
			if (other.jabberId != null) return false;
		}
		else if (!jabberId.equals(other.jabberId)) return false;
		if (peerNames == null)
		{
			if (other.peerNames != null) return false;
		}
		else if (!peerNames.equals(other.peerNames)) return false;
		if (recordOwner == null)
		{
			if (other.recordOwner != null) return false;
		}
		else if (!recordOwner.equals(other.recordOwner)) return false;
		if (recorded == null)
		{
			if (other.recorded != null) return false;
		}
		else if (!recorded.equals(other.recorded)) return false;
		if (recorder == null)
		{
			if (other.recorder != null) return false;
		}
		else if (!recorder.equals(other.recorder)) return false;
		if (referenceOfConsultation == null)
		{
			if (other.referenceOfConsultation != null) return false;
		}
		else if (!referenceOfConsultation.equals(other.referenceOfConsultation)) return false;
		if (sipCallIds == null)
		{
			if (other.sipCallIds != null) return false;
		}
		else if (!sipCallIds.equals(other.sipCallIds)) return false;
		if (state != other.state) return false;
		if (timestamp == null)
		{
			if (other.timestamp != null) return false;
		}
		else if (!timestamp.equals(other.timestamp)) return false;
		return true;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		return "Call [id=" + id + ", state=" + state + ", timestamp=" + timestamp + ", calledNumber="
			+ calledNumber + ", calledName=" + calledName + ", callerNumber=" + callerNumber + ", callerName="
			+ callerName + ", groupId=" + groupId + ", peerNames=" + peerNames + ", channelNames=" + channelNames
			+ ", sipCallIds=" + sipCallIds + ", caller=" + caller + ", connectedTimestamp=" + connectedTimestamp
			+ ", recorded=" + recorded + ", recorder=" + recorder + ", recordOwner=" + recordOwner
			+ ", avatarHash=" + avatarHash + ", doorlineCamUrl=" + doorlineCamUrl + ", doorlineDtmfCode="
			+ doorlineDtmfCode + ", doorlineImageProviderId=" + doorlineImageProviderId + ", forwardType="
			+ forwardType + ", forwarderCallerIdName=" + forwarderCallerIdName + ", forwarderCallerIdNumber="
			+ forwarderCallerIdNumber + ", conferenceRoomId=" + conferenceRoomId + ", conferenceAdmin="
			+ conferenceAdmin + ", referenceOfConsultation=" + referenceOfConsultation + ", duration=" + duration
			+ ", consultationCall=" + consultationCall + ", jabberId=" + jabberId + "]";
	}
	
}
