package de.starface.integration.uci.java.v30.types;

import java.io.Serializable;
import java.util.Arrays;

import de.starface.com.rpc.annotation.RpcValue;
import de.starface.com.rpc.annotation.RpcValueObject;
import de.starface.integration.uci.java.v30.messages.requests.UciFunctionKeyRequests;
import de.starface.integration.uci.java.v30.messages.requests.UciUserStateRequests;

/**
 * This object holds the properties of the images as returned by
 * {@link UciFunctionKeyRequests#getImageForKey(String)} or
 * {@link UciUserStateRequests#getAvatarImage()}
 * 
 * @author tilmann
 */
@RpcValueObject
public class Image implements Serializable
{
	private static final long serialVersionUID = 1L;
	
	/** The binary image data */
	@RpcValue
	private byte[] data;
	
	/** The mime type of the image data */
	@RpcValue
	private String mimeType;
	
	/** The SHA-1 hash of the image data */
	@RpcValue
	private String hash;
	
	public byte[] getData()
	{
		return data;
	}
	
	public void setData(byte[] data)
	{
		this.data = data;
	}
	
	public String getMimeType()
	{
		return mimeType;
	}
	
	public void setMimeType(String mimeType)
	{
		this.mimeType = mimeType;
	}
	
	public String getHash()
	{
		return hash;
	}
	
	public void setHash(String hash)
	{
		this.hash = hash;
	}
	
	@Override
	public String toString()
	{
		return "Image [data=<skipped binary data>, mimeType=" + mimeType + ", hash=" + hash + "]";
	}
	
	@Override
	public int hashCode()
	{
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(data);
		result = prime * result + ((hash == null) ? 0 : hash.hashCode());
		result = prime * result + ((mimeType == null) ? 0 : mimeType.hashCode());
		return result;
	}
	
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj) return true;
		if (obj == null) return false;
		if (getClass() != obj.getClass()) return false;
		Image other = (Image) obj;
		if (!Arrays.equals(data, other.data)) return false;
		if (hash == null)
		{
			if (other.hash != null) return false;
		}
		else if (!hash.equals(other.hash)) return false;
		if (mimeType == null)
		{
			if (other.mimeType != null) return false;
		}
		else if (!mimeType.equals(other.mimeType)) return false;
		return true;
	}
}
