package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.com.rpc.annotation.RpcValueTranslation;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.UciServices;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.types.FmcPhone;

/**
 * This interface defines the FMC phone requests provided by the UCI server.
 * 
 * @author shennlich
 */
@RpcInterface(UcpFmcPhoneRequests.SERVICE_NAME)
@UciService(UcpFmcPhoneRequests.SERVICE_NAME)
@RpcValueTranslation(version = UciServices.UCI_VERSION)
public interface UcpFmcPhoneRequests
{
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.fmcPhone";
	
	/**
	 * Saving a new FMC phone or updating one.
	 * 
	 * @param phone
	 * @throws UciException
	 */
	public void saveFmcPhone(FmcPhone phone) throws UciException;
	
	/**
	 * Deleting an FMC phone.
	 * 
	 * @param phone
	 * @throws UciException
	 */
	public void deleateFmcPhone(FmcPhone phone) throws UciException;
	
	/**
	 * Getting all FMC phones of this user.
	 * 
	 * @throws UciException
	 */
	public Map<String, Object> getFmcPhones() throws UciException;
}
