/*
 * $Id: UcpGroupRequests.java 847 2013-11-20 11:18:26Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * ucp/v21/server/UcpServerCommunicationCall.java $
 * 
 * $LastChangedDate: 2013-11-20 12:18:26 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 847 $
 */
package de.starface.integration.uci.java.v30.ucp.messages.requests;

import java.util.List;
import java.util.Map;

import de.starface.com.rpc.annotation.RpcInterface;
import de.starface.integration.uci.java.v30.UciService;
import de.starface.integration.uci.java.v30.exceptions.UciException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalAccessException;
import de.starface.integration.uci.java.v30.exceptions.UciIllegalArgumentException;
import de.starface.integration.uci.java.v30.ucp.exceptions.UcpNotConnectedException;
import de.starface.integration.uci.java.v30.values.GroupSettingProperties;

/**
 * This interface defines the group requests provided by the UCI server.
 * 
 * @author tilmann
 */
@UciService(UcpGroupRequests.SERVICE_NAME)
@RpcInterface(UcpGroupRequests.SERVICE_NAME)
public interface UcpGroupRequests
{
	
	/** The UCI service name used for this interface */
	public static final String SERVICE_NAME = "ucp.v30.requests.group";
	
	/**
	 * This request allows the UCI client to query the group settings of the groups the current user
	 * is a member of.
	 * 
	 * @return a {@link List} with a {@link Map} for each group containing the values for the
	 *         properties defined in {@link GroupSettingProperties}
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if the request with the given parameters is not allowed to the user (
	 *         {@link UciIllegalAccessException}) or if the request is placed while the UCI client
	 *         is not connected ({@link UcpNotConnectedException}).
	 */
	public List<Map<String, Object>> getGroupSettings() throws UciException;
	
	/**
	 * This request saves the given group setting. The value for the properties
	 * {@link GroupSettingProperties#name} and {@link GroupSettingProperties#phoneNumbers} are
	 * ignored. This is read only information given by the server.
	 * 
	 * @param groupSetting
	 *        the setting to be saved
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setGroupSetting(Map<String, Object> groupSetting) throws UciException;
	
	/**
	 * This request saves the given group settings. The value for the properties
	 * {@link GroupSettingProperties#name} and {@link GroupSettingProperties#phoneNumbers} are
	 * ignored. This is read only information given by the server.
	 * 
	 * @param groupSettings
	 *        the settings to be saved
	 * @throws UciException
	 *         if the request is not valid as defined by the UCI specification. This may be the case
	 *         if one of the request's arguments has an illegal value (
	 *         {@link UciIllegalArgumentException}) or if the request with the given parameters is
	 *         not allowed to the user ({@link UciIllegalAccessException}) or if the request is
	 *         placed while the UCI client is not connected ({@link UcpNotConnectedException}).
	 */
	public void setGroupSettings(List<Map<String, Object>> groupSettings) throws UciException;
}
