package de.starface.integration.uci.java.v30.values;

/**
 * This enum defines the different states a call can take. The constants are used for the call
 * property {@link CallProperties#state}.
 * 
 * @author tilmann
 */
public enum CallState
{
	/**
	 * A new call is incoming for the account and has not been further handeled. Especially no
	 * phones are ringing for the call so far.
	 */
	INCOMING,
	
	/**
	 * A new call is outgoing for the account and has not been further handeled. Especially no
	 * phones are ringing for the call so far.
	 */
	OUTGOING,
	
	/**
	 * The identified account is called by someone an it's telephone is ringing.
	 */
	RINGING,
	
	/**
	 * The the identified account is calling someone whose telephone is ringing.
	 */
	RINGBACK,
	
	/**
	 * This state indicates that the two parties of the call have been connected and the
	 * corresponding call is established.
	 */
	CONNECTED,
	
	/**
	 * This state indicates that the call is parked
	 */
	PARKED,
	
	/**
	 * This state indicates that the call is in conference
	 */
	CONFERENCE_ACTIVE,
	
	/**
	 * This state indicates that the call is used to log in or out of a telephone or a group
	 */
	LOGIN_LOGOUT,
	
	/**
	 * This state indicates that the call is used to test the music on hold
	 */
	MUSICONHOLD_TEST,
	
	/**
	 * When a call is linked to the voicemailbox.
	 */
	VOICEMAIL_LINKED,
	
	/**
	 * When a call is linked to the voicemailbox menue.
	 */
	VOICEMAIL_MAIN,
	
	/**
	 * A consultation call.
	 */
	CONSULT,
	
	/**
	 * A consultation call for conference.
	 */
	CONFERENCE_CONSULT,
	
	/**
	 * The account have no active connection to that conference.
	 */
	CONFERENCE_INACTIVE,
	
	/**
	 * Ccbs is activated for that call.
	 */
	CCBS,
	
	/**
	 * When a call reaches this state it is finished independent from its previous state.
	 */
	HANGUP
}
