/*
 * $Id: VoicemailListEntryProperties.java 847 2013-11-20 11:18:26Z dirk $
 * 
 * Copyright (C) by Vertico Software GmbH, Karlsruhe, Germany. All rights reserved.
 * 
 * NOTICE: THIS MATERIAL IS CONSIDERED A TRADE SECRET BY VERTICO SOFTWARE. UNAUTHORIZED ACCESS, USE,
 * REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
 * 
 * created on 06.08.2009 by tilmann
 * 
 * $HeadURL:
 * http://192.168.2.8/repos/eai-uci/java/trunk/ucp/src/de/vertico/starface/integration/uci/
 * v21/VoicemailListEntryProperties.java $
 * 
 * $LastChangedDate: 2013-11-20 12:18:26 +0100 (Mi, 20. Nov 2013) $ $LastChangedBy: dirk $
 * $LastChangedRevision: 847 $
 */
package de.starface.integration.uci.java.v30.values;

import java.util.Date;

import de.starface.integration.uci.java.v30.ucp.messages.requests.UcpMonitorListRequests;

/**
 * This enum holds constants with the property names of the monitor list entries as returned by
 * {@link UcpMonitorListRequests#getMonitorList(Date, Date, String, String, int, int)}
 * 
 * @author tilmann
 */
public enum MonitorListEntryProperties
{
	/** The id of the monitor list entry as {@link String} */
	id(OrderDirection.ASCENDING, String.class),
	
	/** CallerCallerID, if known as {@link String} */
	callercallerid(OrderDirection.ASCENDING, String.class),
	
	/** CalledCallerID, if known as {@link String} */
	calledcallerid(OrderDirection.ASCENDING, String.class),
	
	/** Callbacknumber, if known as {@link String} */
	callbacknumber(OrderDirection.ASCENDING, String.class),
	
	/**
	 * The file of the monitor {@link String}.
	 */
	file(OrderDirection.ASCENDING, String.class),
	
	/**
	 * The start time of the call as {@link java.util.Date}. Per default dates will be ordered
	 * newest first
	 */
	monitortimestart(OrderDirection.DESCENDING, Date.class),
	
	/**
	 * The end time of the call as {@link java.util.Date}. Per default dates will be ordered newest
	 * first
	 */
	monitortimeend(OrderDirection.DESCENDING, Date.class),
	
	/** The duration of the call in milliseconds as {@link Integer} */
	duration(OrderDirection.ASCENDING, Long.class);
	
	private OrderDirection defaultOrderDirection;
	
	private Class<?> type;
	
	private MonitorListEntryProperties(OrderDirection defaultOrderDirection, Class<?> type)
	{
		this.defaultOrderDirection = defaultOrderDirection;
		this.type = type;
	}
	
	/**
	 * @return the default {@link OrderDirection} for the current property
	 */
	public OrderDirection getDefaultOrderDirection()
	{
		return defaultOrderDirection;
	}
	
	/**
	 * @return the java type of the property
	 */
	public Class<?> getType()
	{
		return type;
	}
	
}
